# Name:         xrced.py
# Purpose:      XRC editor, main module
# Author:       Roman Rolinsky <rolinsky@mema.ucl.ac.be>
# Created:      20.08.2001
# RCS-ID:       $Id: xrced.py,v 1.4.2.17 2003/03/21 22:35:49 RD Exp $

"""

xrced -- Simple resource editor for XRC format used by wxWindows/wxPython
         GUI toolkit.

Usage:

    xrced [ -h ] [ -i ] [ -v ] [ XRC-file ]

Options:

    -h          output short usage info and exit

    -i          use international character set instead of translations

    -v          output version info and exit
"""


from globals import *
import os, sys, getopt, re, traceback

# Local modules
from tree import *                      # imports xxx which imports params
from panel import *
from tools import *
# Cleanup recursive import sideeffects, otherwise we can't create undoMan
import undo
undo.ParamPage = ParamPage
undoMan = g.undoMan = UndoManager()

# Set application path for loading resources
if __name__ == '__main__':
    basePath = os.path.dirname(sys.argv[0])
else:
    basePath = os.path.dirname(__file__)

# 1 adds CMD command to Help menu
debug = 0

g.helpText = """\
<HTML><H2>Welcome to XRC<font color="blue">ed</font></H2><H3><font color="green">DON'T PANIC :)</font></H3>
Read this note before clicking on anything!<P>
To start select tree root, then popup menu with your right mouse button,
select "Append Child", and then any command.<P>
Or just press one of the buttons on the tools palette.<P>
Enter XML ID, change properties, create children.<P>
To test your interface select Test command (View menu).<P>
Consult README file for the details.</HTML>
"""

defaultIDs = {xxxPanel:'PANEL', xxxDialog:'DIALOG', xxxFrame:'FRAME',
              xxxMenuBar:'MENUBAR', xxxMenu:'MENU', xxxToolBar:'TOOLBAR'}

################################################################################

# ScrolledMessageDialog - modified from wxPython lib to set fixed-width font
class ScrolledMessageDialog(wxDialog):
    def __init__(self, parent, msg, caption, pos = wxDefaultPosition, size = (500,300)):
        from wxPython.lib.layoutf import Layoutf
        wxDialog.__init__(self, parent, -1, caption, pos, size)
        text = wxTextCtrl(self, -1, msg, wxDefaultPosition,
                             wxDefaultSize, wxTE_MULTILINE | wxTE_READONLY)
        text.SetFont(modernFont)
        dc = wxWindowDC(text)
        # !!! possible bug - GetTextExtent without font returns sysfont dims
        w, h = dc.GetFullTextExtent(' ', modernFont)[:2]
        ok = wxButton(self, wxID_OK, "OK")
        text.SetConstraints(Layoutf('t=t5#1;b=t5#2;l=l5#1;r=r5#1', (self,ok)))
        text.SetSize((w * 80 + 30, h * 40))
        ok.SetConstraints(Layoutf('b=b5#1;x%w50#1;w!80;h!25', (self,)))
        self.SetAutoLayout(True)
        self.Fit()
        self.CenterOnScreen(wxBOTH)

################################################################################

class Frame(wxFrame):
    def __init__(self, pos, size):
        wxFrame.__init__(self, None, -1, '', pos, size)
        global frame
        frame = g.frame = self
        bar = self.CreateStatusBar(2)
        bar.SetStatusWidths([-1, 40])
        self.SetIcon(images.getIconIcon())

        # Idle flag
        self.inIdle = False

        # Make menus
        menuBar = wxMenuBar()

        menu = wxMenu()
        menu.Append(wxID_NEW, '&New\tCtrl-N', 'New file')
        menu.Append(wxID_OPEN, '&Open...\tCtrl-O', 'Open XRC file')
        menu.Append(wxID_SAVE, '&Save\tCtrl-S', 'Save XRC file')
        menu.Append(wxID_SAVEAS, 'Save &As...', 'Save XRC file under different name')
        menu.AppendSeparator()
        menu.Append(wxID_EXIT, '&Quit\tCtrl-Q', 'Exit application')
        menuBar.Append(menu, '&File')

        menu = wxMenu()
        menu.Append(wxID_UNDO, '&Undo\tCtrl-Z', 'Undo')
        menu.Append(wxID_REDO, '&Redo\tCtrl-Y', 'Redo')
        menu.AppendSeparator()
        menu.Append(wxID_CUT, 'Cut\tCtrl-X', 'Cut to the clipboard')
        menu.Append(wxID_COPY, '&Copy\tCtrl-C', 'Copy to the clipboard')
        menu.Append(wxID_PASTE, '&Paste\tCtrl-V', 'Paste from the clipboard')
        self.ID_DELETE = wxNewId()
        menu.Append(self.ID_DELETE, '&Delete\tCtrl-D', 'Delete object')
#        menu.AppendSeparator()
        ID_SELECT = wxNewId()
#        menu.Append(ID_SELECT, '&Select', 'Select object')
        menuBar.Append(menu, '&Edit')

        menu = wxMenu()
        self.ID_EMBED_PANEL = wxNewId()
        menu.Append(self.ID_EMBED_PANEL, '&Embed Panel',
                    'Toggle embedding properties panel in the main window', True)
        menu.Check(self.ID_EMBED_PANEL, conf.embedPanel)
        self.ID_SHOW_TOOLS = wxNewId()
        menu.Append(self.ID_SHOW_TOOLS, 'Show &Tools', 'Toggle tools', True)
        menu.Check(self.ID_SHOW_TOOLS, conf.showTools)
        menu.AppendSeparator()
        self.ID_TEST = wxNewId()
        menu.Append(self.ID_TEST, '&Test\tF5', 'Test window')
        self.ID_REFRESH = wxNewId()
        menu.Append(self.ID_REFRESH, '&Refresh\tCtrl-R', 'Refresh test window')
        self.ID_AUTO_REFRESH = wxNewId()
        menu.Append(self.ID_AUTO_REFRESH, '&Auto-refresh\tCtrl-A',
                    'Toggle auto-refresh mode', True)
        menu.Check(self.ID_AUTO_REFRESH, conf.autoRefresh)
        menuBar.Append(menu, '&View')

        menu = wxMenu()
        menu.Append(wxID_ABOUT, '&About...', 'About XCRed')
        self.ID_README = wxNewId()
        menu.Append(self.ID_README, '&Readme...', 'View the README file')
        if debug:
            self.ID_DEBUG_CMD = wxNewId()
            menu.Append(self.ID_DEBUG_CMD, 'CMD', 'Python command line')
            EVT_MENU(self, self.ID_DEBUG_CMD, self.OnDebugCMD)
        menuBar.Append(menu, '&Help')

        self.menuBar = menuBar
        self.SetMenuBar(menuBar)

        # Create toolbar
        tb = self.CreateToolBar(wxTB_HORIZONTAL | wxNO_BORDER | wxTB_FLAT)
        tb.SetToolBitmapSize((24, 23))
        tb.AddSimpleTool(wxID_NEW, images.getNewBitmap(), 'New', 'New file')
        tb.AddSimpleTool(wxID_OPEN, images.getOpenBitmap(), 'Open', 'Open file')
        tb.AddSimpleTool(wxID_SAVE, images.getSaveBitmap(), 'Save', 'Save file')
        tb.AddControl(wxStaticLine(tb, -1, size=(-1,23), style=wxLI_VERTICAL))
        tb.AddSimpleTool(wxID_UNDO, images.getUndoBitmap(), 'Undo', 'Undo')
        tb.AddSimpleTool(wxID_REDO, images.getRedoBitmap(), 'Redo', 'Redo')
        tb.AddControl(wxStaticLine(tb, -1, size=(-1,23), style=wxLI_VERTICAL))
        tb.AddSimpleTool(wxID_CUT, images.getCutBitmap(), 'Cut', 'Cut')
        tb.AddSimpleTool(wxID_COPY, images.getCopyBitmap(), 'Copy', 'Copy')
        tb.AddSimpleTool(wxID_PASTE, images.getPasteBitmap(), 'Paste', 'Paste')
        tb.AddControl(wxStaticLine(tb, -1, size=(-1,23), style=wxLI_VERTICAL))
        tb.AddSimpleTool(self.ID_TEST, images.getTestBitmap(), 'Test', 'Test window')
        tb.AddSimpleTool(self.ID_REFRESH, images.getRefreshBitmap(),
                         'Refresh', 'Refresh view')
        tb.AddSimpleTool(self.ID_AUTO_REFRESH, images.getAutoRefreshBitmap(),
                         'Auto-refresh', 'Toggle auto-refresh mode', True)
        if wxPlatform == '__WXGTK__':
            tb.AddSeparator()   # otherwise auto-refresh sticks in status line
        tb.ToggleTool(self.ID_AUTO_REFRESH, conf.autoRefresh)
        tb.Realize()
        self.tb = tb
        self.minWidth = tb.GetSize()[0] # minimal width is the size of toolbar

        # File
        EVT_MENU(self, wxID_NEW, self.OnNew)
        EVT_MENU(self, wxID_OPEN, self.OnOpen)
        EVT_MENU(self, wxID_SAVE, self.OnSaveOrSaveAs)
        EVT_MENU(self, wxID_SAVEAS, self.OnSaveOrSaveAs)
        EVT_MENU(self, wxID_EXIT, self.OnExit)
        # Edit
        EVT_MENU(self, wxID_UNDO, self.OnUndo)
        EVT_MENU(self, wxID_REDO, self.OnRedo)
        EVT_MENU(self, wxID_CUT, self.OnCutDelete)
        EVT_MENU(self, wxID_COPY, self.OnCopy)
        EVT_MENU(self, wxID_PASTE, self.OnPaste)
        EVT_MENU(self, self.ID_DELETE, self.OnCutDelete)
        EVT_MENU(self, ID_SELECT, self.OnSelect)
        # View
        EVT_MENU(self, self.ID_EMBED_PANEL, self.OnEmbedPanel)
        EVT_MENU(self, self.ID_SHOW_TOOLS, self.OnShowTools)
        EVT_MENU(self, self.ID_TEST, self.OnTest)
        EVT_MENU(self, self.ID_REFRESH, self.OnRefresh)
        EVT_MENU(self, self.ID_AUTO_REFRESH, self.OnAutoRefresh)
        # Help
        EVT_MENU(self, wxID_ABOUT, self.OnAbout)
        EVT_MENU(self, self.ID_README, self.OnReadme)

        # Update events
        EVT_UPDATE_UI(self, wxID_CUT, self.OnUpdateUI)
        EVT_UPDATE_UI(self, wxID_COPY, self.OnUpdateUI)
        EVT_UPDATE_UI(self, wxID_PASTE, self.OnUpdateUI)
        EVT_UPDATE_UI(self, wxID_UNDO, self.OnUpdateUI)
        EVT_UPDATE_UI(self, wxID_REDO, self.OnUpdateUI)
        EVT_UPDATE_UI(self, self.ID_DELETE, self.OnUpdateUI)
        EVT_UPDATE_UI(self, self.ID_TEST, self.OnUpdateUI)
        EVT_UPDATE_UI(self, self.ID_REFRESH, self.OnUpdateUI)

        # Build interface
        sizer = wxBoxSizer(wxVERTICAL)
        sizer.Add(wxStaticLine(self, -1), 0, wxEXPAND)
        # Horizontal sizer for toolbar and splitter
        self.toolsSizer = sizer1 = wxBoxSizer()
        splitter = wxSplitterWindow(self, -1, style=wxSP_3DSASH)
        self.splitter = splitter
        splitter.SetMinimumPaneSize(100)
        # Create tree
        global tree
        g.tree = tree = XML_Tree(splitter, -1)

        # Init pull-down menu data
        global pullDownMenu
        g.pullDownMenu = pullDownMenu = PullDownMenu(self)

        # Vertical toolbar for GUI buttons
        g.tools = tools = Tools(self)
        tools.Show(conf.showTools)
        if conf.showTools: sizer1.Add(tools, 0, wxEXPAND)

        tree.RegisterKeyEvents()

        # !!! frame styles are broken
        # Miniframe for not embedded mode
        miniFrame = wxFrame(self, -1, 'Properties Panel',
                            (conf.panelX, conf.panelY),
                            (conf.panelWidth, conf.panelHeight))
        self.miniFrame = miniFrame
        sizer2 = wxBoxSizer()
        miniFrame.SetAutoLayout(True)
        miniFrame.SetSizer(sizer2)
        EVT_CLOSE(self.miniFrame, self.OnCloseMiniFrame)
        # Create panel for parameters
        global panel
        if conf.embedPanel:
            panel = Panel(splitter)
            # Set plitter windows
            splitter.SplitVertically(tree, panel, conf.sashPos)
        else:
            panel = Panel(miniFrame)
            sizer2.Add(panel, 1, wxEXPAND)
            miniFrame.Show(True)
            splitter.Initialize(tree)
        sizer1.Add(splitter, 1, wxEXPAND)
        sizer.Add(sizer1, 1, wxEXPAND)
        self.SetAutoLayout(True)
        self.SetSizer(sizer)

        # Initialize
        self.clipboard = None
        self.Clear()

        # Other events
        EVT_IDLE(self, self.OnIdle)
        EVT_CLOSE(self, self.OnCloseWindow)
        EVT_LEFT_DOWN(self, self.OnLeftDown)
        EVT_KEY_DOWN(self, tools.OnKeyDown)
        EVT_KEY_UP(self, tools.OnKeyUp)

    def OnNew(self, evt):
        self.Clear()

    def OnOpen(self, evt):
        if not self.AskSave(): return
        dlg = wxFileDialog(self, 'Open', os.path.dirname(self.dataFile),
                           '', '*.xrc', wxOPEN | wxCHANGE_DIR)
        if dlg.ShowModal() == wxID_OK:
            path = dlg.GetPath()
            self.SetStatusText('Loading...')
            wxYield()
            wxBeginBusyCursor()
            if self.Open(path):
                self.SetStatusText('Data loaded')
            else:
                self.SetStatusText('Failed')
            wxEndBusyCursor()
        dlg.Destroy()

    def OnSaveOrSaveAs(self, evt):
        if evt.GetId() == wxID_SAVEAS or not self.dataFile:
            if self.dataFile: defaultName = ''
            else: defaultName = 'UNTITLED.xrc'
            dlg = wxFileDialog(self, 'Save As', os.path.dirname(self.dataFile),
                               defaultName, '*.xrc',
                               wxSAVE | wxOVERWRITE_PROMPT | wxCHANGE_DIR)
            if dlg.ShowModal() == wxID_OK:
                path = dlg.GetPath()
                dlg.Destroy()
            else:
                dlg.Destroy()
                return
        else:
            path = self.dataFile
        self.SetStatusText('Saving...')
        wxYield()
        wxBeginBusyCursor()
        try:
            self.Save(path)
            self.dataFile = path
            self.SetStatusText('Data saved')
        except IOError:
            self.SetStatusText('Failed')
        wxEndBusyCursor()

    def OnExit(self, evt):
        self.Close()

    def OnUndo(self, evt):
        # Extra check to not mess with idle updating
        if undoMan.CanUndo():
            undoMan.Undo()

    def OnRedo(self, evt):
        if undoMan.CanRedo():
            undoMan.Redo()

    def OnCopy(self, evt):
        selected = tree.selection
        if not selected: return         # key pressed event
        xxx = tree.GetPyData(selected)
        self.clipboard = xxx.element.cloneNode(True)
        self.SetStatusText('Copied')

    def OnPaste(self, evt):
        selected = tree.selection
        if not selected: return         # key pressed event
        # For pasting with Ctrl pressed
        if evt.GetId() == pullDownMenu.ID_PASTE_SIBLING: appendChild = False
        else: appendChild = not tree.NeedInsert(selected)
        xxx = tree.GetPyData(selected)
        if not appendChild:
            # If has next item, insert, else append to parent
            nextItem = tree.GetNextSibling(selected)
            parentLeaf = tree.GetItemParent(selected)
        # Expanded container (must have children)
        elif tree.IsExpanded(selected) and tree.GetChildrenCount(selected, False):
            # Insert as first child
            nextItem = tree.GetFirstChild(selected, 0)[0]
            parentLeaf = selected
        else:
            # No children or unexpanded item - appendChild stays True
            nextItem = wxTreeItemId()   # no next item
            parentLeaf = selected
        parent = tree.GetPyData(parentLeaf).treeObject()

        # Create a copy of clipboard element
        elem = self.clipboard.cloneNode(True)
        # Tempopary xxx object to test things
        xxx = MakeXXXFromDOM(parent, elem)

        # Check compatibility
        error = False
        # Top-level
        x = xxx.treeObject()
        if x.__class__ in [xxxDialog, xxxFrame, xxxMenuBar]:
            # Top-level classes
            if parent.__class__ != xxxMainNode: error = True
        elif x.__class__ == xxxToolBar:
            # Toolbar can be top-level of child of panel or frame
            if parent.__class__ not in [xxxMainNode, xxxPanel, xxxFrame]: error = True
        elif x.__class__ == xxxPanel and parent.__class__ == xxxMainNode:
            pass
        elif x.__class__ == xxxSpacer:
            if not parent.isSizer: error = True
        elif x.__class__ == xxxSeparator:
            if not parent.__class__ in [xxxMenu, xxxToolBar]: error = True
        elif x.__class__ == xxxTool:
            if parent.__class__ != xxxToolBar: error = True
        elif x.__class__ == xxxMenu:
            if not parent.__class__ in [xxxMainNode, xxxMenuBar, xxxMenu]: error = True
        elif x.__class__ == xxxMenuItem:
            if not parent.__class__ in [xxxMenuBar, xxxMenu]: error = True
        elif x.isSizer and parent.__class__ == xxxNotebook: error = True
        else:                           # normal controls can be almost anywhere
            if parent.__class__ == xxxMainNode or \
               parent.__class__ in [xxxMenuBar, xxxMenu]: error = True
        if error:
            if parent.__class__ == xxxMainNode: parentClass = 'root'
            else: parentClass = parent.className
            wxLogError('Incompatible parent/child: parent is %s, child is %s!' %
                       (parentClass, x.className))
            return

        # Check parent and child relationships.
        # If parent is sizer or notebook, child is of wrong class or
        # parent is normal window, child is child container then detach child.
        isChildContainer = isinstance(xxx, xxxChildContainer)
        if isChildContainer and \
           ((parent.isSizer and not isinstance(xxx, xxxSizerItem)) or \
            (isinstance(parent, xxxNotebook) and not isinstance(xxx, xxxNotebookPage)) or \
           not (parent.isSizer or isinstance(parent, xxxNotebook))):
            elem.removeChild(xxx.child.element) # detach child
            elem.unlink()           # delete child container
            elem = xxx.child.element # replace
            # This may help garbage collection
            xxx.child.parent = None
            isChildContainer = False
        # Parent is sizer or notebook, child is not child container
        if parent.isSizer and not isChildContainer and not isinstance(xxx, xxxSpacer):
            # Create sizer item element
            sizerItemElem = MakeEmptyDOM('sizeritem')
            sizerItemElem.appendChild(elem)
            elem = sizerItemElem
        elif isinstance(parent, xxxNotebook) and not isChildContainer:
            pageElem = MakeEmptyDOM('notebookpage')
            pageElem.appendChild(elem)
            elem = pageElem
        # Insert new node, register undo
        newItem = tree.InsertNode(parentLeaf, parent, elem, nextItem)
        undoMan.RegisterUndo(UndoPasteCreate(parentLeaf, parent, newItem, selected))
        # Scroll to show new item (!!! redundant?)
        tree.EnsureVisible(newItem)
        tree.SelectItem(newItem)
        if not tree.IsVisible(newItem):
            tree.ScrollTo(newItem)
            tree.Refresh()
        # Update view?
        if g.testWin and tree.IsHighlatable(newItem):
            if conf.autoRefresh:
                tree.needUpdate = True
                tree.pendingHighLight = newItem
            else:
                tree.pendingHighLight = None
        self.modified = True
        self.SetStatusText('Pasted')

    def OnCutDelete(self, evt):
        selected = tree.selection
        if not selected: return         # key pressed event
        # Undo info
        if evt.GetId() == wxID_CUT:
            self.lastOp = 'CUT'
            status = 'Removed to clipboard'
        else:
            self.lastOp = 'DELETE'
            status = 'Deleted'
        # Delete testWin?
        if g.testWin:
            # If deleting top-level item, delete testWin
            if selected == g.testWin.item:
                g.testWin.Destroy()
                g.testWin = None
            else:
                # Remove highlight, update testWin
                if g.testWin.highLight:
                    g.testWin.highLight.Remove()
                tree.needUpdate = True
        # Prepare undo data
        panel.Apply()
        index = tree.ItemFullIndex(selected)
        parent = tree.GetPyData(tree.GetItemParent(selected)).treeObject()
        elem = tree.RemoveLeaf(selected)
        undoMan.RegisterUndo(UndoCutDelete(index, parent, elem))
        if evt.GetId() == wxID_CUT:
            if self.clipboard: self.clipboard.unlink()
            self.clipboard = elem.cloneNode(True)
        tree.pendingHighLight = None
        tree.Unselect()
        panel.Clear()
        self.modified = True
        self.SetStatusText(status)

    def OnSelect(self, evt):
        print >> sys.stderr, 'Xperimental function!'
        wxYield()
        self.SetCursor(wxCROSS_CURSOR)
        self.CaptureMouse()

    def OnLeftDown(self, evt):
        pos = evt.GetPosition()
        self.SetCursor(wxNullCursor)
        self.ReleaseMouse()

    def OnEmbedPanel(self, evt):
        conf.embedPanel = evt.IsChecked()
        if conf.embedPanel:
            # Remember last dimentions
            conf.panelX, conf.panelY = self.miniFrame.GetPosition()
            conf.panelWidth, conf.panelHeight = self.miniFrame.GetSize()
            size = self.GetSize()
            pos = self.GetPosition()
            sizePanel = panel.GetSize()
            panel.Reparent(self.splitter)
            self.miniFrame.GetSizer().RemoveWindow(panel)
            wxYield()
            # Widen
            self.SetDimensions(pos.x, pos.y, size.x + sizePanel.x, size.y)
            self.splitter.SplitVertically(tree, panel, conf.sashPos)
            self.miniFrame.Show(False)
        else:
            conf.sashPos = self.splitter.GetSashPosition()
            pos = self.GetPosition()
            size = self.GetSize()
            sizePanel = panel.GetSize()
            self.splitter.Unsplit(panel)
            sizer = self.miniFrame.GetSizer()
            panel.Reparent(self.miniFrame)
            panel.Show(True)
            sizer.Add(panel, 1, wxEXPAND)
            self.miniFrame.Show(True)
            self.miniFrame.SetDimensions(conf.panelX, conf.panelY,
                                         conf.panelWidth, conf.panelHeight)
            wxYield()
            # Reduce width
            self.SetDimensions(pos.x, pos.y,
                               max(size.x - sizePanel.x, self.minWidth), size.y)

    def OnShowTools(self, evt):
        conf.showTools = evt.IsChecked()
        g.tools.Show(conf.showTools)
        if conf.showTools:
            self.toolsSizer.Prepend(g.tools, 0, wxEXPAND)
        else:
            self.toolsSizer.Remove(g.tools)
        self.toolsSizer.Layout()

    def OnTest(self, evt):
        if not tree.selection: return   # key pressed event
        tree.ShowTestWindow(tree.selection)

    def OnRefresh(self, evt):
        # If modified, apply first
        selection = tree.selection
        if selection:
            xxx = tree.GetPyData(selection)
            if xxx and panel.IsModified():
                tree.Apply(xxx, selection)
        if g.testWin:
            # (re)create
            tree.CreateTestWin(g.testWin.item)
        panel.modified = False
        tree.needUpdate = False

    def OnAutoRefresh(self, evt):
        conf.autoRefresh = evt.IsChecked()
        self.menuBar.Check(self.ID_AUTO_REFRESH, conf.autoRefresh)
        self.tb.ToggleTool(self.ID_AUTO_REFRESH, conf.autoRefresh)

    def OnAbout(self, evt):
        str = '''\
XRCed version %s

(c) Roman Rolinsky <rollrom@users.sourceforge.net>
Homepage: http://xrced.sourceforge.net\
''' % version
        dlg = wxMessageDialog(self, str, 'About XRCed', wxOK | wxCENTRE)
        dlg.ShowModal()
        dlg.Destroy()

    def OnReadme(self, evt):
        text = open(os.path.join(basePath, 'README.txt'), 'r').read()
        dlg = ScrolledMessageDialog(self, text, "XRCed README")
        dlg.ShowModal()
        dlg.Destroy()

    # Simple emulation of python command line
    def OnDebugCMD(self, evt):
        import traceback
        while 1:
            try:
                exec raw_input('C:\> ')
            except EOFError:
                print '^D'
                break
            except:
                (etype, value, tb) =sys.exc_info()
                tblist =traceback.extract_tb(tb)[1:]
                msg =' '.join(traceback.format_exception_only(etype, value)
                        +traceback.format_list(tblist))
                print msg

    def OnCreate(self, evt):
        selected = tree.selection
        if tree.ctrl: appendChild = False
        else: appendChild = not tree.NeedInsert(selected)
        xxx = tree.GetPyData(selected)
        if not appendChild:
            # If insert before
            if tree.shift:
                # If has previous item, insert after it, else append to parent
                nextItem = selected
                parentLeaf = tree.GetItemParent(selected)
            else:
                # If has next item, insert, else append to parent
                nextItem = tree.GetNextSibling(selected)
                parentLeaf = tree.GetItemParent(selected)
        # Expanded container (must have children)
        elif tree.shift and tree.IsExpanded(selected) \
           and tree.GetChildrenCount(selected, False):
            nextItem = tree.GetFirstChild(selected, 0)[0]
            parentLeaf = selected
        else:
            nextItem = wxTreeItemId()
            parentLeaf = selected
        parent = tree.GetPyData(parentLeaf)
        if parent.hasChild: parent = parent.child

        # Create element
        className = pullDownMenu.createMap[evt.GetId()]
        xxx = MakeEmptyXXX(parent, className)

        # Set default name for top-level windows
        if parent.__class__ == xxxMainNode:
            cl = xxx.treeObject().__class__
            frame.maxIDs[cl] += 1
            xxx.treeObject().name = '%s%d' % (defaultIDs[cl], frame.maxIDs[cl])
            xxx.treeObject().element.setAttribute('name', xxx.treeObject().name)

        # Insert new node, register undo
        elem = xxx.element
        newItem = tree.InsertNode(parentLeaf, parent, elem, nextItem)
        undoMan.RegisterUndo(UndoPasteCreate(parentLeaf, parent, newItem, selected))
        tree.EnsureVisible(newItem)
        tree.SelectItem(newItem)
        if not tree.IsVisible(newItem):
            tree.ScrollTo(newItem)
            tree.Refresh()
        # Update view?
        if g.testWin and tree.IsHighlatable(newItem):
            if conf.autoRefresh:
                tree.needUpdate = True
                tree.pendingHighLight = newItem
            else:
                tree.pendingHighLight = None
        tree.SetFocus()

    # Expand/collapse subtree
    def OnExpand(self, evt):
        if tree.selection: tree.ExpandAll(tree.selection)
        else: tree.ExpandAll(tree.root)
    def OnCollapse(self, evt):
        if tree.selection: tree.CollapseAll(tree.selection)
        else: tree.CollapseAll(tree.root)

    def OnPullDownHighlight(self, evt):
        menuId = evt.GetMenuId()
        if menuId != -1:
            menu = evt.GetEventObject()
            help = menu.GetHelpString(menuId)
            self.SetStatusText(help)
        else:
            self.SetStatusText('')

    def OnUpdateUI(self, evt):
        if evt.GetId() in [wxID_CUT, wxID_COPY, self.ID_DELETE]:
            evt.Enable(tree.selection is not None and tree.selection != tree.root)
        elif evt.GetId() == wxID_PASTE:
            evt.Enable((self.clipboard and tree.selection) != None)
        elif evt.GetId() == self.ID_TEST:
            evt.Enable(tree.selection is not None and tree.selection != tree.root)
        elif evt.GetId() == wxID_UNDO:  evt.Enable(undoMan.CanUndo())
        elif evt.GetId() == wxID_REDO:  evt.Enable(undoMan.CanRedo())

    def OnIdle(self, evt):
        if self.inIdle: return          # Recursive call protection
        self.inIdle = True
        if tree.needUpdate:
            if conf.autoRefresh:
                if g.testWin:
                    self.SetStatusText('Refreshing test window...')
                    # (re)create
                    tree.CreateTestWin(g.testWin.item)
                    wxYield()
                    self.SetStatusText('')
                tree.needUpdate = False
        elif tree.pendingHighLight:
            tree.HighLight(tree.pendingHighLight)
        else:
            evt.Skip()
        self.inIdle = False

    # We don't let close panel window
    def OnCloseMiniFrame(self, evt):
        return

    def OnCloseWindow(self, evt):
        if not self.AskSave(): return
        if g.testWin: g.testWin.Destroy()
        # Destroy cached windows
        panel.cacheParent.Destroy()
        if not panel.GetPageCount() == 2:
            panel.page2.Destroy()
        conf.x, conf.y = self.GetPosition()
        conf.width, conf.height = self.GetSize()
        if conf.embedPanel:
            conf.sashPos = self.splitter.GetSashPosition()
        else:
            conf.panelX, conf.panelY = self.miniFrame.GetPosition()
            conf.panelWidth, conf.panelHeight = self.miniFrame.GetSize()
        evt.Skip()

    def Clear(self):
        self.dataFile = ''
        if self.clipboard:
            self.clipboard.unlink()
            self.clipboard = None
        undoMan.Clear()
        self.modified = False
        tree.Clear()
        panel.Clear()
        if g.testWin:
            g.testWin.Destroy()
            g.testWin = None
        self.SetTitle(progname)
        # Numbers for new controls
        self.maxIDs = {}
        self.maxIDs[xxxPanel] = self.maxIDs[xxxDialog] = self.maxIDs[xxxFrame] = \
        self.maxIDs[xxxMenuBar] = self.maxIDs[xxxMenu] = self.maxIDs[xxxToolBar] = 0

    def Open(self, path):
        if not os.path.exists(path):
            wxLogError('File does not exists: %s' % path)
            return False
        # Try to read the file
        try:
            f = open(path)
            self.Clear()
            # Parse first line to get encoding (!! hack, I don't know a better way)
            line = f.readline()
            mo = re.match(r'^<\?xml ([^<>]* )?encoding="(?P<encd>[^<>].*)"\?>', line)
            # Build wx tree
            f.seek(0)
            dom = minidom.parse(f)
            # Set encoding global variable and document encoding property
            import xxx
            if mo:
                dom.encoding = xxx.currentEncoding = mo.group('encd')
            else:
                xxx.currentEncoding = 'iso-8859-1'
                dom.encoding = ''
            f.close()
            # Change dir
            dir = os.path.dirname(path)
            if dir: os.chdir(dir)
            tree.SetData(dom)
            self.dataFile = path
            self.SetTitle(progname + ': ' + os.path.basename(path))
        except:
            # Nice exception printing
            inf = sys.exc_info()
            wxLogError(traceback.format_exception(inf[0], inf[1], None)[-1])
            wxLogError('Error reading file: %s' % path)
            return False
        return True

    def Indent(self, node, indent = 0):
        # Copy child list because it will change soon
        children = node.childNodes[:]
        # Main node doesn't need to be indented
        if indent:
            text = self.domCopy.createTextNode('\n' + ' ' * indent)
            node.parentNode.insertBefore(text, node)
        if children:
            # Append newline after last child, except for text nodes
            if children[-1].nodeType == minidom.Node.ELEMENT_NODE:
                text = self.domCopy.createTextNode('\n' + ' ' * indent)
                node.appendChild(text)
            # Indent children which are elements
            for n in children:
                if n.nodeType == minidom.Node.ELEMENT_NODE:
                    self.Indent(n, indent + 2)

    def Save(self, path):
        try:
            # Apply changes
            if tree.selection and panel.IsModified():
                self.OnRefresh(wxCommandEvent())
            f = open(path, 'w')
            # Make temporary copy for formatting it
            # !!! We can't clone dom node, it works only once
            #self.domCopy = tree.dom.cloneNode(True)
            self.domCopy = MyDocument()
            mainNode = self.domCopy.appendChild(tree.mainNode.cloneNode(True))
            self.Indent(mainNode)
            self.domCopy.writexml(f, encoding=tree.rootObj.params['encoding'].value())
            f.close()
            self.domCopy.unlink()
            self.domCopy = None
            self.modified = False
            panel.SetModified(False)
        except:
            wxLogError('Error writing file: %s' % path)
            raise

    def AskSave(self):
        if not (self.modified or panel.IsModified()): return True
        flags = wxICON_EXCLAMATION | wxYES_NO | wxCANCEL | wxCENTRE
        dlg = wxMessageDialog( self, 'File is modified. Save before exit?',
                               'Save before too late?', flags )
        say = dlg.ShowModal()
        dlg.Destroy()
        if say == wxID_YES:
            self.OnSaveOrSaveAs(wxCommandEvent(wxID_SAVE))
            # If save was successful, modified flag is unset
            if not self.modified: return True
        elif say == wxID_NO:
            self.modified = False
            panel.SetModified(False)
            return True
        return False

    def SaveUndo(self):
        pass                            # !!!

################################################################################

def usage():
    print >> sys.stderr, 'usage: xrced [-dhlv] [file]'

class App(wxApp):
    def OnInit(self):
        global debug
        # Process comand-line
        try:
            opts = args = []  #give empty values in case of exception
            opts, args = getopt.getopt(sys.argv[1:], 'dhiv')
        except getopt.GetoptError:
            if wxPlatform != '__WXMAC__': # macs have some extra parameters
                print >> sys.stderr, 'Unknown option'
                usage()
                sys.exit(1)
        for o,a in opts:
            if o == '-h':
                usage()
                sys.exit(0)
            elif o == '-d':
                debug = True
            elif o == '-i':
                g.xmlFlags &= ~wxXRC_USE_LOCALE
            elif o == '-v':
                print 'XRCed version', version
                sys.exit(0)

        self.SetAppName('xrced')
        # Settings
        global conf
        conf = g.conf = wxConfig(style = wxCONFIG_USE_LOCAL_FILE)
        conf.autoRefresh = conf.ReadInt('autorefresh', True)
        pos = conf.ReadInt('x', -1), conf.ReadInt('y', -1)
        size = conf.ReadInt('width', 800), conf.ReadInt('height', 600)
        conf.embedPanel = conf.ReadInt('embedPanel', True)
        conf.showTools = conf.ReadInt('showTools', True)
        conf.sashPos = conf.ReadInt('sashPos', 200)
        if not conf.embedPanel:
            conf.panelX = conf.ReadInt('panelX', -1)
            conf.panelY = conf.ReadInt('panelY', -1)
        else:
            conf.panelX = conf.panelY = -1
        conf.panelWidth = conf.ReadInt('panelWidth', 200)
        conf.panelHeight = conf.ReadInt('panelHeight', 200)
        conf.panic = not conf.HasEntry('nopanic')
        # Add handlers
        wxFileSystem_AddHandler(wxMemoryFSHandler())
        wxInitAllImageHandlers()
        # Create main frame
        frame = Frame(pos, size)
        frame.Show(True)
        # Load resources from XRC file (!!! should be transformed to .py later?)
        frame.res = wxXmlResource('')
        frame.res.Load(os.path.join(basePath, 'xrced.xrc'))

        # Load file after showing
        if args:
            conf.panic = False
            frame.open = frame.Open(args[0])

        return True

    def OnExit(self):
        # Write config
        global conf
        wc = wxConfigBase_Get()
        wc.WriteInt('autorefresh', conf.autoRefresh)
        wc.WriteInt('x', conf.x)
        wc.WriteInt('y', conf.y)
        wc.WriteInt('width', conf.width)
        wc.WriteInt('height', conf.height)
        wc.WriteInt('embedPanel', conf.embedPanel)
        wc.WriteInt('showTools', conf.showTools)
        if not conf.embedPanel:
            wc.WriteInt('panelX', conf.panelX)
            wc.WriteInt('panelY', conf.panelY)
        wc.WriteInt('sashPos', conf.sashPos)
        wc.WriteInt('panelWidth', conf.panelWidth)
        wc.WriteInt('panelHeight', conf.panelHeight)
        wc.WriteInt('nopanic', True)
        wc.Flush()

def main():
    app = App(0, useBestVisual=False)
    app.MainLoop()
    app.OnExit()
    global conf
    del conf

if __name__ == '__main__':
    main()



# This file was created automatically by SWIG.
import utilsc
import wx
class wxConfigBasePtr :
    Type_Unknown = utilsc.wxConfigBase_Type_Unknown
    Type_String = utilsc.wxConfigBase_Type_String
    Type_Boolean = utilsc.wxConfigBase_Type_Boolean
    Type_Integer = utilsc.wxConfigBase_Type_Integer
    Type_Float = utilsc.wxConfigBase_Type_Float
    def __init__(self,this):
        self.this = this
        self.thisown = 0
    def __del__(self, delfunc=utilsc.delete_wxConfigBase):
        if self.thisown == 1:
            try:
                delfunc(self)
            except:
                pass
    def DeleteAll(self, *_args, **_kwargs):
        val = apply(utilsc.wxConfigBase_DeleteAll,(self,) + _args, _kwargs)
        return val
    def DeleteEntry(self, *_args, **_kwargs):
        val = apply(utilsc.wxConfigBase_DeleteEntry,(self,) + _args, _kwargs)
        return val
    def DeleteGroup(self, *_args, **_kwargs):
        val = apply(utilsc.wxConfigBase_DeleteGroup,(self,) + _args, _kwargs)
        return val
    def Exists(self, *_args, **_kwargs):
        val = apply(utilsc.wxConfigBase_Exists,(self,) + _args, _kwargs)
        return val
    def Flush(self, *_args, **_kwargs):
        val = apply(utilsc.wxConfigBase_Flush,(self,) + _args, _kwargs)
        return val
    def GetAppName(self, *_args, **_kwargs):
        val = apply(utilsc.wxConfigBase_GetAppName,(self,) + _args, _kwargs)
        return val
    def GetFirstGroup(self, *_args, **_kwargs):
        val = apply(utilsc.wxConfigBase_GetFirstGroup,(self,) + _args, _kwargs)
        return val
    def GetFirstEntry(self, *_args, **_kwargs):
        val = apply(utilsc.wxConfigBase_GetFirstEntry,(self,) + _args, _kwargs)
        return val
    def GetNextGroup(self, *_args, **_kwargs):
        val = apply(utilsc.wxConfigBase_GetNextGroup,(self,) + _args, _kwargs)
        return val
    def GetNextEntry(self, *_args, **_kwargs):
        val = apply(utilsc.wxConfigBase_GetNextEntry,(self,) + _args, _kwargs)
        return val
    def GetNumberOfEntries(self, *_args, **_kwargs):
        val = apply(utilsc.wxConfigBase_GetNumberOfEntries,(self,) + _args, _kwargs)
        return val
    def GetNumberOfGroups(self, *_args, **_kwargs):
        val = apply(utilsc.wxConfigBase_GetNumberOfGroups,(self,) + _args, _kwargs)
        return val
    def GetPath(self, *_args, **_kwargs):
        val = apply(utilsc.wxConfigBase_GetPath,(self,) + _args, _kwargs)
        return val
    def GetVendorName(self, *_args, **_kwargs):
        val = apply(utilsc.wxConfigBase_GetVendorName,(self,) + _args, _kwargs)
        return val
    def HasEntry(self, *_args, **_kwargs):
        val = apply(utilsc.wxConfigBase_HasEntry,(self,) + _args, _kwargs)
        return val
    def HasGroup(self, *_args, **_kwargs):
        val = apply(utilsc.wxConfigBase_HasGroup,(self,) + _args, _kwargs)
        return val
    def IsExpandingEnvVars(self, *_args, **_kwargs):
        val = apply(utilsc.wxConfigBase_IsExpandingEnvVars,(self,) + _args, _kwargs)
        return val
    def IsRecordingDefaults(self, *_args, **_kwargs):
        val = apply(utilsc.wxConfigBase_IsRecordingDefaults,(self,) + _args, _kwargs)
        return val
    def Read(self, *_args, **_kwargs):
        val = apply(utilsc.wxConfigBase_Read,(self,) + _args, _kwargs)
        return val
    def ReadInt(self, *_args, **_kwargs):
        val = apply(utilsc.wxConfigBase_ReadInt,(self,) + _args, _kwargs)
        return val
    def ReadFloat(self, *_args, **_kwargs):
        val = apply(utilsc.wxConfigBase_ReadFloat,(self,) + _args, _kwargs)
        return val
    def ReadBool(self, *_args, **_kwargs):
        val = apply(utilsc.wxConfigBase_ReadBool,(self,) + _args, _kwargs)
        return val
    def SetExpandEnvVars(self, *_args, **_kwargs):
        val = apply(utilsc.wxConfigBase_SetExpandEnvVars,(self,) + _args, _kwargs)
        return val
    def SetPath(self, *_args, **_kwargs):
        val = apply(utilsc.wxConfigBase_SetPath,(self,) + _args, _kwargs)
        return val
    def SetRecordDefaults(self, *_args, **_kwargs):
        val = apply(utilsc.wxConfigBase_SetRecordDefaults,(self,) + _args, _kwargs)
        return val
    def SetAppName(self, *_args, **_kwargs):
        val = apply(utilsc.wxConfigBase_SetAppName,(self,) + _args, _kwargs)
        return val
    def SetVendorName(self, *_args, **_kwargs):
        val = apply(utilsc.wxConfigBase_SetVendorName,(self,) + _args, _kwargs)
        return val
    def SetStyle(self, *_args, **_kwargs):
        val = apply(utilsc.wxConfigBase_SetStyle,(self,) + _args, _kwargs)
        return val
    def GetStyle(self, *_args, **_kwargs):
        val = apply(utilsc.wxConfigBase_GetStyle,(self,) + _args, _kwargs)
        return val
    def Write(self, *_args, **_kwargs):
        val = apply(utilsc.wxConfigBase_Write,(self,) + _args, _kwargs)
        return val
    def WriteInt(self, *_args, **_kwargs):
        val = apply(utilsc.wxConfigBase_WriteInt,(self,) + _args, _kwargs)
        return val
    def WriteFloat(self, *_args, **_kwargs):
        val = apply(utilsc.wxConfigBase_WriteFloat,(self,) + _args, _kwargs)
        return val
    def WriteBool(self, *_args, **_kwargs):
        val = apply(utilsc.wxConfigBase_WriteBool,(self,) + _args, _kwargs)
        return val
    def GetEntryType(self, *_args, **_kwargs):
        val = apply(utilsc.wxConfigBase_GetEntryType,(self,) + _args, _kwargs)
        return val
    def RenameEntry(self, *_args, **_kwargs):
        val = apply(utilsc.wxConfigBase_RenameEntry,(self,) + _args, _kwargs)
        return val
    def RenameGroup(self, *_args, **_kwargs):
        val = apply(utilsc.wxConfigBase_RenameGroup,(self,) + _args, _kwargs)
        return val
    def ExpandEnvVars(self, *_args, **_kwargs):
        val = apply(utilsc.wxConfigBase_ExpandEnvVars,(self,) + _args, _kwargs)
        return val
    def __repr__(self):
        return "<C wxConfigBase instance at %s>" % (self.this,)
class wxConfigBase(wxConfigBasePtr):
    def __init__(self,this):
        self.this = this




class wxConfigPtr(wxConfigBasePtr):
    def __init__(self,this):
        self.this = this
        self.thisown = 0
    def __del__(self, delfunc=utilsc.delete_wxConfig):
        if self.thisown == 1:
            try:
                delfunc(self)
            except:
                pass
    def __repr__(self):
        return "<C wxConfig instance at %s>" % (self.this,)
class wxConfig(wxConfigPtr):
    def __init__(self,*_args,**_kwargs):
        self.this = apply(utilsc.new_wxConfig,_args,_kwargs)
        self.thisown = 1




class wxFileConfigPtr(wxConfigBasePtr):
    def __init__(self,this):
        self.this = this
        self.thisown = 0
    def __del__(self, delfunc=utilsc.delete_wxFileConfig):
        if self.thisown == 1:
            try:
                delfunc(self)
            except:
                pass
    def __repr__(self):
        return "<C wxFileConfig instance at %s>" % (self.this,)
class wxFileConfig(wxFileConfigPtr):
    def __init__(self,*_args,**_kwargs):
        self.this = apply(utilsc.new_wxFileConfig,_args,_kwargs)
        self.thisown = 1




class wxDateTimePtr :
    Local = utilsc.wxDateTime_Local
    GMT_12 = utilsc.wxDateTime_GMT_12
    GMT_11 = utilsc.wxDateTime_GMT_11
    GMT_10 = utilsc.wxDateTime_GMT_10
    GMT_9 = utilsc.wxDateTime_GMT_9
    GMT_8 = utilsc.wxDateTime_GMT_8
    GMT_7 = utilsc.wxDateTime_GMT_7
    GMT_6 = utilsc.wxDateTime_GMT_6
    GMT_5 = utilsc.wxDateTime_GMT_5
    GMT_4 = utilsc.wxDateTime_GMT_4
    GMT_3 = utilsc.wxDateTime_GMT_3
    GMT_2 = utilsc.wxDateTime_GMT_2
    GMT_1 = utilsc.wxDateTime_GMT_1
    GMT0 = utilsc.wxDateTime_GMT0
    GMT1 = utilsc.wxDateTime_GMT1
    GMT2 = utilsc.wxDateTime_GMT2
    GMT3 = utilsc.wxDateTime_GMT3
    GMT4 = utilsc.wxDateTime_GMT4
    GMT5 = utilsc.wxDateTime_GMT5
    GMT6 = utilsc.wxDateTime_GMT6
    GMT7 = utilsc.wxDateTime_GMT7
    GMT8 = utilsc.wxDateTime_GMT8
    GMT9 = utilsc.wxDateTime_GMT9
    GMT10 = utilsc.wxDateTime_GMT10
    GMT11 = utilsc.wxDateTime_GMT11
    GMT12 = utilsc.wxDateTime_GMT12
    WET = utilsc.wxDateTime_WET
    WEST = utilsc.wxDateTime_WEST
    CET = utilsc.wxDateTime_CET
    CEST = utilsc.wxDateTime_CEST
    EET = utilsc.wxDateTime_EET
    EEST = utilsc.wxDateTime_EEST
    MSK = utilsc.wxDateTime_MSK
    MSD = utilsc.wxDateTime_MSD
    AST = utilsc.wxDateTime_AST
    ADT = utilsc.wxDateTime_ADT
    EST = utilsc.wxDateTime_EST
    EDT = utilsc.wxDateTime_EDT
    CST = utilsc.wxDateTime_CST
    CDT = utilsc.wxDateTime_CDT
    MST = utilsc.wxDateTime_MST
    MDT = utilsc.wxDateTime_MDT
    PST = utilsc.wxDateTime_PST
    PDT = utilsc.wxDateTime_PDT
    HST = utilsc.wxDateTime_HST
    AKST = utilsc.wxDateTime_AKST
    AKDT = utilsc.wxDateTime_AKDT
    A_WST = utilsc.wxDateTime_A_WST
    A_CST = utilsc.wxDateTime_A_CST
    A_EST = utilsc.wxDateTime_A_EST
    A_ESST = utilsc.wxDateTime_A_ESST
    UTC = utilsc.wxDateTime_UTC
    Gregorian = utilsc.wxDateTime_Gregorian
    Julian = utilsc.wxDateTime_Julian
    Country_Unknown = utilsc.wxDateTime_Country_Unknown
    Country_Default = utilsc.wxDateTime_Country_Default
    Country_WesternEurope_Start = utilsc.wxDateTime_Country_WesternEurope_Start
    Country_EEC = utilsc.wxDateTime_Country_EEC
    France = utilsc.wxDateTime_France
    Germany = utilsc.wxDateTime_Germany
    UK = utilsc.wxDateTime_UK
    Country_WesternEurope_End = utilsc.wxDateTime_Country_WesternEurope_End
    Russia = utilsc.wxDateTime_Russia
    USA = utilsc.wxDateTime_USA
    Jan = utilsc.wxDateTime_Jan
    Feb = utilsc.wxDateTime_Feb
    Mar = utilsc.wxDateTime_Mar
    Apr = utilsc.wxDateTime_Apr
    May = utilsc.wxDateTime_May
    Jun = utilsc.wxDateTime_Jun
    Jul = utilsc.wxDateTime_Jul
    Aug = utilsc.wxDateTime_Aug
    Sep = utilsc.wxDateTime_Sep
    Oct = utilsc.wxDateTime_Oct
    Nov = utilsc.wxDateTime_Nov
    Dec = utilsc.wxDateTime_Dec
    Inv_Month = utilsc.wxDateTime_Inv_Month
    Sun = utilsc.wxDateTime_Sun
    Mon = utilsc.wxDateTime_Mon
    Tue = utilsc.wxDateTime_Tue
    Wed = utilsc.wxDateTime_Wed
    Thu = utilsc.wxDateTime_Thu
    Fri = utilsc.wxDateTime_Fri
    Sat = utilsc.wxDateTime_Sat
    Inv_WeekDay = utilsc.wxDateTime_Inv_WeekDay
    Inv_Year = utilsc.wxDateTime_Inv_Year
    Name_Full = utilsc.wxDateTime_Name_Full
    Name_Abbr = utilsc.wxDateTime_Name_Abbr
    Default_First = utilsc.wxDateTime_Default_First
    Monday_First = utilsc.wxDateTime_Monday_First
    Sunday_First = utilsc.wxDateTime_Sunday_First
    def __init__(self,this):
        self.this = this
        self.thisown = 0
    def __del__(self, delfunc=utilsc.delete_wxDateTime):
        if self.thisown == 1:
            try:
                delfunc(self)
            except:
                pass
    def SetToCurrent(self, *_args, **_kwargs):
        val = apply(utilsc.wxDateTime_SetToCurrent,(self,) + _args, _kwargs)
        if val: val = wxDateTimePtr(val) 
        return val
    def SetTimeT(self, *_args, **_kwargs):
        val = apply(utilsc.wxDateTime_SetTimeT,(self,) + _args, _kwargs)
        if val: val = wxDateTimePtr(val) 
        return val
    def SetJDN(self, *_args, **_kwargs):
        val = apply(utilsc.wxDateTime_SetJDN,(self,) + _args, _kwargs)
        if val: val = wxDateTimePtr(val) 
        return val
    def SetHMS(self, *_args, **_kwargs):
        val = apply(utilsc.wxDateTime_SetHMS,(self,) + _args, _kwargs)
        if val: val = wxDateTimePtr(val) 
        return val
    def Set(self, *_args, **_kwargs):
        val = apply(utilsc.wxDateTime_Set,(self,) + _args, _kwargs)
        if val: val = wxDateTimePtr(val) 
        return val
    def ResetTime(self, *_args, **_kwargs):
        val = apply(utilsc.wxDateTime_ResetTime,(self,) + _args, _kwargs)
        if val: val = wxDateTimePtr(val) 
        return val
    def SetYear(self, *_args, **_kwargs):
        val = apply(utilsc.wxDateTime_SetYear,(self,) + _args, _kwargs)
        if val: val = wxDateTimePtr(val) 
        return val
    def SetMonth(self, *_args, **_kwargs):
        val = apply(utilsc.wxDateTime_SetMonth,(self,) + _args, _kwargs)
        if val: val = wxDateTimePtr(val) 
        return val
    def SetDay(self, *_args, **_kwargs):
        val = apply(utilsc.wxDateTime_SetDay,(self,) + _args, _kwargs)
        if val: val = wxDateTimePtr(val) 
        return val
    def SetHour(self, *_args, **_kwargs):
        val = apply(utilsc.wxDateTime_SetHour,(self,) + _args, _kwargs)
        if val: val = wxDateTimePtr(val) 
        return val
    def SetMinute(self, *_args, **_kwargs):
        val = apply(utilsc.wxDateTime_SetMinute,(self,) + _args, _kwargs)
        if val: val = wxDateTimePtr(val) 
        return val
    def SetSecond(self, *_args, **_kwargs):
        val = apply(utilsc.wxDateTime_SetSecond,(self,) + _args, _kwargs)
        if val: val = wxDateTimePtr(val) 
        return val
    def SetMillisecond(self, *_args, **_kwargs):
        val = apply(utilsc.wxDateTime_SetMillisecond,(self,) + _args, _kwargs)
        if val: val = wxDateTimePtr(val) 
        return val
    def SetToWeekDayInSameWeek(self, *_args, **_kwargs):
        val = apply(utilsc.wxDateTime_SetToWeekDayInSameWeek,(self,) + _args, _kwargs)
        if val: val = wxDateTimePtr(val) 
        return val
    def GetWeekDayInSameWeek(self, *_args, **_kwargs):
        val = apply(utilsc.wxDateTime_GetWeekDayInSameWeek,(self,) + _args, _kwargs)
        if val: val = wxDateTimePtr(val) ; val.thisown = 1
        return val
    def SetToNextWeekDay(self, *_args, **_kwargs):
        val = apply(utilsc.wxDateTime_SetToNextWeekDay,(self,) + _args, _kwargs)
        if val: val = wxDateTimePtr(val) 
        return val
    def GetNextWeekDay(self, *_args, **_kwargs):
        val = apply(utilsc.wxDateTime_GetNextWeekDay,(self,) + _args, _kwargs)
        if val: val = wxDateTimePtr(val) ; val.thisown = 1
        return val
    def SetToPrevWeekDay(self, *_args, **_kwargs):
        val = apply(utilsc.wxDateTime_SetToPrevWeekDay,(self,) + _args, _kwargs)
        if val: val = wxDateTimePtr(val) 
        return val
    def GetPrevWeekDay(self, *_args, **_kwargs):
        val = apply(utilsc.wxDateTime_GetPrevWeekDay,(self,) + _args, _kwargs)
        if val: val = wxDateTimePtr(val) ; val.thisown = 1
        return val
    def SetToWeekDay(self, *_args, **_kwargs):
        val = apply(utilsc.wxDateTime_SetToWeekDay,(self,) + _args, _kwargs)
        return val
    def SetToLastWeekDay(self, *_args, **_kwargs):
        val = apply(utilsc.wxDateTime_SetToLastWeekDay,(self,) + _args, _kwargs)
        return val
    def GetLastWeekDay(self, *_args, **_kwargs):
        val = apply(utilsc.wxDateTime_GetLastWeekDay,(self,) + _args, _kwargs)
        if val: val = wxDateTimePtr(val) ; val.thisown = 1
        return val
    def SetToTheWeek(self, *_args, **_kwargs):
        val = apply(utilsc.wxDateTime_SetToTheWeek,(self,) + _args, _kwargs)
        return val
    def GetWeek(self, *_args, **_kwargs):
        val = apply(utilsc.wxDateTime_GetWeek,(self,) + _args, _kwargs)
        if val: val = wxDateTimePtr(val) ; val.thisown = 1
        return val
    def SetToLastMonthDay(self, *_args, **_kwargs):
        val = apply(utilsc.wxDateTime_SetToLastMonthDay,(self,) + _args, _kwargs)
        if val: val = wxDateTimePtr(val) 
        return val
    def GetLastMonthDay(self, *_args, **_kwargs):
        val = apply(utilsc.wxDateTime_GetLastMonthDay,(self,) + _args, _kwargs)
        if val: val = wxDateTimePtr(val) ; val.thisown = 1
        return val
    def SetToYearDay(self, *_args, **_kwargs):
        val = apply(utilsc.wxDateTime_SetToYearDay,(self,) + _args, _kwargs)
        if val: val = wxDateTimePtr(val) 
        return val
    def GetYearDay(self, *_args, **_kwargs):
        val = apply(utilsc.wxDateTime_GetYearDay,(self,) + _args, _kwargs)
        if val: val = wxDateTimePtr(val) ; val.thisown = 1
        return val
    def GetJulianDayNumber(self, *_args, **_kwargs):
        val = apply(utilsc.wxDateTime_GetJulianDayNumber,(self,) + _args, _kwargs)
        return val
    def GetJDN(self, *_args, **_kwargs):
        val = apply(utilsc.wxDateTime_GetJDN,(self,) + _args, _kwargs)
        return val
    def GetModifiedJulianDayNumber(self, *_args, **_kwargs):
        val = apply(utilsc.wxDateTime_GetModifiedJulianDayNumber,(self,) + _args, _kwargs)
        return val
    def GetMJD(self, *_args, **_kwargs):
        val = apply(utilsc.wxDateTime_GetMJD,(self,) + _args, _kwargs)
        return val
    def GetRataDie(self, *_args, **_kwargs):
        val = apply(utilsc.wxDateTime_GetRataDie,(self,) + _args, _kwargs)
        return val
    def ToTimezone(self, *_args, **_kwargs):
        val = apply(utilsc.wxDateTime_ToTimezone,(self,) + _args, _kwargs)
        if val: val = wxDateTimePtr(val) ; val.thisown = 1
        return val
    def MakeTimezone(self, *_args, **_kwargs):
        val = apply(utilsc.wxDateTime_MakeTimezone,(self,) + _args, _kwargs)
        if val: val = wxDateTimePtr(val) 
        return val
    def ToGMT(self, *_args, **_kwargs):
        val = apply(utilsc.wxDateTime_ToGMT,(self,) + _args, _kwargs)
        if val: val = wxDateTimePtr(val) ; val.thisown = 1
        return val
    def MakeGMT(self, *_args, **_kwargs):
        val = apply(utilsc.wxDateTime_MakeGMT,(self,) + _args, _kwargs)
        if val: val = wxDateTimePtr(val) 
        return val
    def IsDST(self, *_args, **_kwargs):
        val = apply(utilsc.wxDateTime_IsDST,(self,) + _args, _kwargs)
        return val
    def IsValid(self, *_args, **_kwargs):
        val = apply(utilsc.wxDateTime_IsValid,(self,) + _args, _kwargs)
        return val
    def GetTicks(self, *_args, **_kwargs):
        val = apply(utilsc.wxDateTime_GetTicks,(self,) + _args, _kwargs)
        return val
    def GetYear(self, *_args, **_kwargs):
        val = apply(utilsc.wxDateTime_GetYear,(self,) + _args, _kwargs)
        return val
    def GetMonth(self, *_args, **_kwargs):
        val = apply(utilsc.wxDateTime_GetMonth,(self,) + _args, _kwargs)
        return val
    def GetDay(self, *_args, **_kwargs):
        val = apply(utilsc.wxDateTime_GetDay,(self,) + _args, _kwargs)
        return val
    def GetWeekDay(self, *_args, **_kwargs):
        val = apply(utilsc.wxDateTime_GetWeekDay,(self,) + _args, _kwargs)
        return val
    def GetHour(self, *_args, **_kwargs):
        val = apply(utilsc.wxDateTime_GetHour,(self,) + _args, _kwargs)
        return val
    def GetMinute(self, *_args, **_kwargs):
        val = apply(utilsc.wxDateTime_GetMinute,(self,) + _args, _kwargs)
        return val
    def GetSecond(self, *_args, **_kwargs):
        val = apply(utilsc.wxDateTime_GetSecond,(self,) + _args, _kwargs)
        return val
    def GetMillisecond(self, *_args, **_kwargs):
        val = apply(utilsc.wxDateTime_GetMillisecond,(self,) + _args, _kwargs)
        return val
    def GetDayOfYear(self, *_args, **_kwargs):
        val = apply(utilsc.wxDateTime_GetDayOfYear,(self,) + _args, _kwargs)
        return val
    def GetWeekOfYear(self, *_args, **_kwargs):
        val = apply(utilsc.wxDateTime_GetWeekOfYear,(self,) + _args, _kwargs)
        return val
    def GetWeekOfMonth(self, *_args, **_kwargs):
        val = apply(utilsc.wxDateTime_GetWeekOfMonth,(self,) + _args, _kwargs)
        return val
    def IsWorkDay(self, *_args, **_kwargs):
        val = apply(utilsc.wxDateTime_IsWorkDay,(self,) + _args, _kwargs)
        return val
    def IsEqualTo(self, *_args, **_kwargs):
        val = apply(utilsc.wxDateTime_IsEqualTo,(self,) + _args, _kwargs)
        return val
    def IsEarlierThan(self, *_args, **_kwargs):
        val = apply(utilsc.wxDateTime_IsEarlierThan,(self,) + _args, _kwargs)
        return val
    def IsLaterThan(self, *_args, **_kwargs):
        val = apply(utilsc.wxDateTime_IsLaterThan,(self,) + _args, _kwargs)
        return val
    def IsStrictlyBetween(self, *_args, **_kwargs):
        val = apply(utilsc.wxDateTime_IsStrictlyBetween,(self,) + _args, _kwargs)
        return val
    def IsBetween(self, *_args, **_kwargs):
        val = apply(utilsc.wxDateTime_IsBetween,(self,) + _args, _kwargs)
        return val
    def IsSameDate(self, *_args, **_kwargs):
        val = apply(utilsc.wxDateTime_IsSameDate,(self,) + _args, _kwargs)
        return val
    def IsSameTime(self, *_args, **_kwargs):
        val = apply(utilsc.wxDateTime_IsSameTime,(self,) + _args, _kwargs)
        return val
    def IsEqualUpTo(self, *_args, **_kwargs):
        val = apply(utilsc.wxDateTime_IsEqualUpTo,(self,) + _args, _kwargs)
        return val
    def AddTS(self, *_args, **_kwargs):
        val = apply(utilsc.wxDateTime_AddTS,(self,) + _args, _kwargs)
        if val: val = wxDateTimePtr(val) 
        return val
    def AddDS(self, *_args, **_kwargs):
        val = apply(utilsc.wxDateTime_AddDS,(self,) + _args, _kwargs)
        if val: val = wxDateTimePtr(val) 
        return val
    def SubtractTS(self, *_args, **_kwargs):
        val = apply(utilsc.wxDateTime_SubtractTS,(self,) + _args, _kwargs)
        if val: val = wxDateTimePtr(val) 
        return val
    def SubtractDS(self, *_args, **_kwargs):
        val = apply(utilsc.wxDateTime_SubtractDS,(self,) + _args, _kwargs)
        if val: val = wxDateTimePtr(val) 
        return val
    def Subtract(self, *_args, **_kwargs):
        val = apply(utilsc.wxDateTime_Subtract,(self,) + _args, _kwargs)
        if val: val = wxTimeSpanPtr(val) ; val.thisown = 1
        return val
    def __add__TS(self, *_args, **_kwargs):
        val = apply(utilsc.wxDateTime___add__TS,(self,) + _args, _kwargs)
        if val: val = wxDateTimePtr(val) ; val.thisown = 1
        return val
    def __add__DS(self, *_args, **_kwargs):
        val = apply(utilsc.wxDateTime___add__DS,(self,) + _args, _kwargs)
        if val: val = wxDateTimePtr(val) ; val.thisown = 1
        return val
    def __sub__DT(self, *_args, **_kwargs):
        val = apply(utilsc.wxDateTime___sub__DT,(self,) + _args, _kwargs)
        if val: val = wxTimeSpanPtr(val) ; val.thisown = 1
        return val
    def __sub__TS(self, *_args, **_kwargs):
        val = apply(utilsc.wxDateTime___sub__TS,(self,) + _args, _kwargs)
        if val: val = wxDateTimePtr(val) ; val.thisown = 1
        return val
    def __sub__DS(self, *_args, **_kwargs):
        val = apply(utilsc.wxDateTime___sub__DS,(self,) + _args, _kwargs)
        if val: val = wxDateTimePtr(val) ; val.thisown = 1
        return val
    def __cmp__(self, *_args, **_kwargs):
        val = apply(utilsc.wxDateTime___cmp__,(self,) + _args, _kwargs)
        return val
    def ParseRfc822Date(self, *_args, **_kwargs):
        val = apply(utilsc.wxDateTime_ParseRfc822Date,(self,) + _args, _kwargs)
        return val
    def ParseFormat(self, *_args, **_kwargs):
        val = apply(utilsc.wxDateTime_ParseFormat,(self,) + _args, _kwargs)
        return val
    def ParseDateTime(self, *_args, **_kwargs):
        val = apply(utilsc.wxDateTime_ParseDateTime,(self,) + _args, _kwargs)
        return val
    def ParseDate(self, *_args, **_kwargs):
        val = apply(utilsc.wxDateTime_ParseDate,(self,) + _args, _kwargs)
        return val
    def ParseTime(self, *_args, **_kwargs):
        val = apply(utilsc.wxDateTime_ParseTime,(self,) + _args, _kwargs)
        return val
    def Format(self, *_args, **_kwargs):
        val = apply(utilsc.wxDateTime_Format,(self,) + _args, _kwargs)
        return val
    def FormatDate(self, *_args, **_kwargs):
        val = apply(utilsc.wxDateTime_FormatDate,(self,) + _args, _kwargs)
        return val
    def FormatTime(self, *_args, **_kwargs):
        val = apply(utilsc.wxDateTime_FormatTime,(self,) + _args, _kwargs)
        return val
    def FormatISODate(self, *_args, **_kwargs):
        val = apply(utilsc.wxDateTime_FormatISODate,(self,) + _args, _kwargs)
        return val
    def FormatISOTime(self, *_args, **_kwargs):
        val = apply(utilsc.wxDateTime_FormatISOTime,(self,) + _args, _kwargs)
        return val
    def __repr__(self):
        return "<C wxDateTime instance at %s>" % (self.this,)
    
    def __add__(self, other):
        if isinstance(other, wxTimeSpanPtr):
            return self.__add__TS(other)
        if isinstance(other, wxDateSpanPtr):
            return self.__add__DS(other)
        raise TypeError, 'Invalid r.h.s. type for __add__'
    def __sub__(self, other):
        if isinstance(other, wxDateTimePtr):
            return self.__sub__DT(other)
        if isinstance(other, wxTimeSpanPtr):
            return self.__sub__TS(other)
        if isinstance(other, wxDateSpanPtr):
            return self.__sub__DS(other)
        raise TypeError, 'Invalid r.h.s. type for __sub__'

    
    def __repr__(self):
        return '<wxDateTime: \"%s\" at %s>' % ( self.Format(), self.this)
    def __str__(self):
        return self.Format()

class wxDateTime(wxDateTimePtr):
    def __init__(self,*_args,**_kwargs):
        self.this = apply(utilsc.new_wxDateTime,_args,_kwargs)
        self.thisown = 1



def wxDateTimeFromTimeT(*_args,**_kwargs):
    val = wxDateTimePtr(apply(utilsc.new_wxDateTimeFromTimeT,_args,_kwargs))
    val.thisown = 1
    return val

def wxDateTimeFromJDN(*_args,**_kwargs):
    val = wxDateTimePtr(apply(utilsc.new_wxDateTimeFromJDN,_args,_kwargs))
    val.thisown = 1
    return val

def wxDateTimeFromHMS(*_args,**_kwargs):
    val = wxDateTimePtr(apply(utilsc.new_wxDateTimeFromHMS,_args,_kwargs))
    val.thisown = 1
    return val

def wxDateTimeFromDMY(*_args,**_kwargs):
    val = wxDateTimePtr(apply(utilsc.new_wxDateTimeFromDMY,_args,_kwargs))
    val.thisown = 1
    return val


class wxTimeSpanPtr :
    def __init__(self,this):
        self.this = this
        self.thisown = 0
    def __del__(self, delfunc=utilsc.delete_wxTimeSpan):
        if self.thisown == 1:
            try:
                delfunc(self)
            except:
                pass
    def Add(self, *_args, **_kwargs):
        val = apply(utilsc.wxTimeSpan_Add,(self,) + _args, _kwargs)
        if val: val = wxTimeSpanPtr(val) 
        return val
    def Subtract(self, *_args, **_kwargs):
        val = apply(utilsc.wxTimeSpan_Subtract,(self,) + _args, _kwargs)
        if val: val = wxTimeSpanPtr(val) 
        return val
    def Multiply(self, *_args, **_kwargs):
        val = apply(utilsc.wxTimeSpan_Multiply,(self,) + _args, _kwargs)
        if val: val = wxTimeSpanPtr(val) 
        return val
    def Neg(self, *_args, **_kwargs):
        val = apply(utilsc.wxTimeSpan_Neg,(self,) + _args, _kwargs)
        if val: val = wxTimeSpanPtr(val) 
        return val
    def Abs(self, *_args, **_kwargs):
        val = apply(utilsc.wxTimeSpan_Abs,(self,) + _args, _kwargs)
        if val: val = wxTimeSpanPtr(val) ; val.thisown = 1
        return val
    def __add__(self, *_args, **_kwargs):
        val = apply(utilsc.wxTimeSpan___add__,(self,) + _args, _kwargs)
        if val: val = wxTimeSpanPtr(val) ; val.thisown = 1
        return val
    def __sub__(self, *_args, **_kwargs):
        val = apply(utilsc.wxTimeSpan___sub__,(self,) + _args, _kwargs)
        if val: val = wxTimeSpanPtr(val) ; val.thisown = 1
        return val
    def __mul__(self, *_args, **_kwargs):
        val = apply(utilsc.wxTimeSpan___mul__,(self,) + _args, _kwargs)
        if val: val = wxTimeSpanPtr(val) ; val.thisown = 1
        return val
    def __rmul__(self, *_args, **_kwargs):
        val = apply(utilsc.wxTimeSpan___rmul__,(self,) + _args, _kwargs)
        if val: val = wxTimeSpanPtr(val) ; val.thisown = 1
        return val
    def __neg__(self, *_args, **_kwargs):
        val = apply(utilsc.wxTimeSpan___neg__,(self,) + _args, _kwargs)
        if val: val = wxTimeSpanPtr(val) ; val.thisown = 1
        return val
    def __cmp__(self, *_args, **_kwargs):
        val = apply(utilsc.wxTimeSpan___cmp__,(self,) + _args, _kwargs)
        return val
    def IsNull(self, *_args, **_kwargs):
        val = apply(utilsc.wxTimeSpan_IsNull,(self,) + _args, _kwargs)
        return val
    def IsPositive(self, *_args, **_kwargs):
        val = apply(utilsc.wxTimeSpan_IsPositive,(self,) + _args, _kwargs)
        return val
    def IsNegative(self, *_args, **_kwargs):
        val = apply(utilsc.wxTimeSpan_IsNegative,(self,) + _args, _kwargs)
        return val
    def IsEqualTo(self, *_args, **_kwargs):
        val = apply(utilsc.wxTimeSpan_IsEqualTo,(self,) + _args, _kwargs)
        return val
    def IsLongerThan(self, *_args, **_kwargs):
        val = apply(utilsc.wxTimeSpan_IsLongerThan,(self,) + _args, _kwargs)
        return val
    def IsShorterThan(self, *_args, **_kwargs):
        val = apply(utilsc.wxTimeSpan_IsShorterThan,(self,) + _args, _kwargs)
        return val
    def GetWeeks(self, *_args, **_kwargs):
        val = apply(utilsc.wxTimeSpan_GetWeeks,(self,) + _args, _kwargs)
        return val
    def GetDays(self, *_args, **_kwargs):
        val = apply(utilsc.wxTimeSpan_GetDays,(self,) + _args, _kwargs)
        return val
    def GetHours(self, *_args, **_kwargs):
        val = apply(utilsc.wxTimeSpan_GetHours,(self,) + _args, _kwargs)
        return val
    def GetMinutes(self, *_args, **_kwargs):
        val = apply(utilsc.wxTimeSpan_GetMinutes,(self,) + _args, _kwargs)
        return val
    def GetSeconds(self, *_args, **_kwargs):
        val = apply(utilsc.wxTimeSpan_GetSeconds,(self,) + _args, _kwargs)
        return val
    def GetMilliseconds(self, *_args, **_kwargs):
        val = apply(utilsc.wxTimeSpan_GetMilliseconds,(self,) + _args, _kwargs)
        return val
    def Format(self, *_args, **_kwargs):
        val = apply(utilsc.wxTimeSpan_Format,(self,) + _args, _kwargs)
        return val
    def __repr__(self):
        return "<C wxTimeSpan instance at %s>" % (self.this,)
class wxTimeSpan(wxTimeSpanPtr):
    def __init__(self,*_args,**_kwargs):
        self.this = apply(utilsc.new_wxTimeSpan,_args,_kwargs)
        self.thisown = 1




class wxDateSpanPtr :
    def __init__(self,this):
        self.this = this
        self.thisown = 0
    def __del__(self, delfunc=utilsc.delete_wxDateSpan):
        if self.thisown == 1:
            try:
                delfunc(self)
            except:
                pass
    def SetYears(self, *_args, **_kwargs):
        val = apply(utilsc.wxDateSpan_SetYears,(self,) + _args, _kwargs)
        if val: val = wxDateSpanPtr(val) 
        return val
    def SetMonths(self, *_args, **_kwargs):
        val = apply(utilsc.wxDateSpan_SetMonths,(self,) + _args, _kwargs)
        if val: val = wxDateSpanPtr(val) 
        return val
    def SetWeeks(self, *_args, **_kwargs):
        val = apply(utilsc.wxDateSpan_SetWeeks,(self,) + _args, _kwargs)
        if val: val = wxDateSpanPtr(val) 
        return val
    def SetDays(self, *_args, **_kwargs):
        val = apply(utilsc.wxDateSpan_SetDays,(self,) + _args, _kwargs)
        if val: val = wxDateSpanPtr(val) 
        return val
    def GetYears(self, *_args, **_kwargs):
        val = apply(utilsc.wxDateSpan_GetYears,(self,) + _args, _kwargs)
        return val
    def GetMonths(self, *_args, **_kwargs):
        val = apply(utilsc.wxDateSpan_GetMonths,(self,) + _args, _kwargs)
        return val
    def GetWeeks(self, *_args, **_kwargs):
        val = apply(utilsc.wxDateSpan_GetWeeks,(self,) + _args, _kwargs)
        return val
    def GetDays(self, *_args, **_kwargs):
        val = apply(utilsc.wxDateSpan_GetDays,(self,) + _args, _kwargs)
        return val
    def GetTotalDays(self, *_args, **_kwargs):
        val = apply(utilsc.wxDateSpan_GetTotalDays,(self,) + _args, _kwargs)
        return val
    def Add(self, *_args, **_kwargs):
        val = apply(utilsc.wxDateSpan_Add,(self,) + _args, _kwargs)
        if val: val = wxDateSpanPtr(val) 
        return val
    def Subtract(self, *_args, **_kwargs):
        val = apply(utilsc.wxDateSpan_Subtract,(self,) + _args, _kwargs)
        if val: val = wxDateSpanPtr(val) 
        return val
    def Neg(self, *_args, **_kwargs):
        val = apply(utilsc.wxDateSpan_Neg,(self,) + _args, _kwargs)
        if val: val = wxDateSpanPtr(val) 
        return val
    def Multiply(self, *_args, **_kwargs):
        val = apply(utilsc.wxDateSpan_Multiply,(self,) + _args, _kwargs)
        if val: val = wxDateSpanPtr(val) 
        return val
    def __add__(self, *_args, **_kwargs):
        val = apply(utilsc.wxDateSpan___add__,(self,) + _args, _kwargs)
        if val: val = wxDateSpanPtr(val) ; val.thisown = 1
        return val
    def __sub__(self, *_args, **_kwargs):
        val = apply(utilsc.wxDateSpan___sub__,(self,) + _args, _kwargs)
        if val: val = wxDateSpanPtr(val) ; val.thisown = 1
        return val
    def __mul__(self, *_args, **_kwargs):
        val = apply(utilsc.wxDateSpan___mul__,(self,) + _args, _kwargs)
        if val: val = wxDateSpanPtr(val) ; val.thisown = 1
        return val
    def __rmul__(self, *_args, **_kwargs):
        val = apply(utilsc.wxDateSpan___rmul__,(self,) + _args, _kwargs)
        if val: val = wxDateSpanPtr(val) ; val.thisown = 1
        return val
    def __neg__(self, *_args, **_kwargs):
        val = apply(utilsc.wxDateSpan___neg__,(self,) + _args, _kwargs)
        if val: val = wxDateSpanPtr(val) ; val.thisown = 1
        return val
    def __repr__(self):
        return "<C wxDateSpan instance at %s>" % (self.this,)
class wxDateSpan(wxDateSpanPtr):
    def __init__(self,*_args,**_kwargs):
        self.this = apply(utilsc.new_wxDateSpan,_args,_kwargs)
        self.thisown = 1






#-------------- FUNCTION WRAPPERS ------------------

wxGetLocalTime = utilsc.wxGetLocalTime

wxGetUTCTime = utilsc.wxGetUTCTime

wxGetCurrentTime = utilsc.wxGetCurrentTime

wxGetLocalTimeMillis = utilsc.wxGetLocalTimeMillis

def wxConfigBase_Set(*_args, **_kwargs):
    val = apply(utilsc.wxConfigBase_Set,_args,_kwargs)
    if val: val = wxConfigBasePtr(val)
    return val

def wxConfigBase_Get(*_args, **_kwargs):
    val = apply(utilsc.wxConfigBase_Get,_args,_kwargs)
    if val: val = wxConfigBasePtr(val)
    return val

def wxConfigBase_Create(*_args, **_kwargs):
    val = apply(utilsc.wxConfigBase_Create,_args,_kwargs)
    if val: val = wxConfigBasePtr(val)
    return val

wxConfigBase_DontCreateOnDemand = utilsc.wxConfigBase_DontCreateOnDemand

wxDateTime_SetCountry = utilsc.wxDateTime_SetCountry

wxDateTime_GetCountry = utilsc.wxDateTime_GetCountry

wxDateTime_IsWestEuropeanCountry = utilsc.wxDateTime_IsWestEuropeanCountry

wxDateTime_GetCurrentYear = utilsc.wxDateTime_GetCurrentYear

wxDateTime_ConvertYearToBC = utilsc.wxDateTime_ConvertYearToBC

wxDateTime_GetCurrentMonth = utilsc.wxDateTime_GetCurrentMonth

wxDateTime_IsLeapYear = utilsc.wxDateTime_IsLeapYear

wxDateTime_GetCentury = utilsc.wxDateTime_GetCentury

wxDateTime_GetNumberOfDaysinYear = utilsc.wxDateTime_GetNumberOfDaysinYear

wxDateTime_GetNumberOfDaysInMonth = utilsc.wxDateTime_GetNumberOfDaysInMonth

wxDateTime_GetMonthName = utilsc.wxDateTime_GetMonthName

wxDateTime_GetWeekDayName = utilsc.wxDateTime_GetWeekDayName

wxDateTime_GetAmPmStrings = utilsc.wxDateTime_GetAmPmStrings

wxDateTime_IsDSTApplicable = utilsc.wxDateTime_IsDSTApplicable

def wxDateTime_GetBeginDST(*_args, **_kwargs):
    val = apply(utilsc.wxDateTime_GetBeginDST,_args,_kwargs)
    if val: val = wxDateTimePtr(val); val.thisown = 1
    return val

def wxDateTime_GetEndDST(*_args, **_kwargs):
    val = apply(utilsc.wxDateTime_GetEndDST,_args,_kwargs)
    if val: val = wxDateTimePtr(val); val.thisown = 1
    return val

def wxDateTime_Now(*_args, **_kwargs):
    val = apply(utilsc.wxDateTime_Now,_args,_kwargs)
    if val: val = wxDateTimePtr(val); val.thisown = 1
    return val

def wxDateTime_UNow(*_args, **_kwargs):
    val = apply(utilsc.wxDateTime_UNow,_args,_kwargs)
    if val: val = wxDateTimePtr(val); val.thisown = 1
    return val

def wxDateTime_Today(*_args, **_kwargs):
    val = apply(utilsc.wxDateTime_Today,_args,_kwargs)
    if val: val = wxDateTimePtr(val); val.thisown = 1
    return val

def wxTimeSpan_Seconds(*_args, **_kwargs):
    val = apply(utilsc.wxTimeSpan_Seconds,_args,_kwargs)
    if val: val = wxTimeSpanPtr(val); val.thisown = 1
    return val

def wxTimeSpan_Second(*_args, **_kwargs):
    val = apply(utilsc.wxTimeSpan_Second,_args,_kwargs)
    if val: val = wxTimeSpanPtr(val); val.thisown = 1
    return val

def wxTimeSpan_Minutes(*_args, **_kwargs):
    val = apply(utilsc.wxTimeSpan_Minutes,_args,_kwargs)
    if val: val = wxTimeSpanPtr(val); val.thisown = 1
    return val

def wxTimeSpan_Minute(*_args, **_kwargs):
    val = apply(utilsc.wxTimeSpan_Minute,_args,_kwargs)
    if val: val = wxTimeSpanPtr(val); val.thisown = 1
    return val

def wxTimeSpan_Hours(*_args, **_kwargs):
    val = apply(utilsc.wxTimeSpan_Hours,_args,_kwargs)
    if val: val = wxTimeSpanPtr(val); val.thisown = 1
    return val

def wxTimeSpan_Hour(*_args, **_kwargs):
    val = apply(utilsc.wxTimeSpan_Hour,_args,_kwargs)
    if val: val = wxTimeSpanPtr(val); val.thisown = 1
    return val

def wxTimeSpan_Days(*_args, **_kwargs):
    val = apply(utilsc.wxTimeSpan_Days,_args,_kwargs)
    if val: val = wxTimeSpanPtr(val); val.thisown = 1
    return val

def wxTimeSpan_Day(*_args, **_kwargs):
    val = apply(utilsc.wxTimeSpan_Day,_args,_kwargs)
    if val: val = wxTimeSpanPtr(val); val.thisown = 1
    return val

def wxTimeSpan_Weeks(*_args, **_kwargs):
    val = apply(utilsc.wxTimeSpan_Weeks,_args,_kwargs)
    if val: val = wxTimeSpanPtr(val); val.thisown = 1
    return val

def wxTimeSpan_Week(*_args, **_kwargs):
    val = apply(utilsc.wxTimeSpan_Week,_args,_kwargs)
    if val: val = wxTimeSpanPtr(val); val.thisown = 1
    return val

def wxDateSpan_Days(*_args, **_kwargs):
    val = apply(utilsc.wxDateSpan_Days,_args,_kwargs)
    if val: val = wxDateSpanPtr(val); val.thisown = 1
    return val

def wxDateSpan_Day(*_args, **_kwargs):
    val = apply(utilsc.wxDateSpan_Day,_args,_kwargs)
    if val: val = wxDateSpanPtr(val); val.thisown = 1
    return val

def wxDateSpan_Weeks(*_args, **_kwargs):
    val = apply(utilsc.wxDateSpan_Weeks,_args,_kwargs)
    if val: val = wxDateSpanPtr(val); val.thisown = 1
    return val

def wxDateSpan_Week(*_args, **_kwargs):
    val = apply(utilsc.wxDateSpan_Week,_args,_kwargs)
    if val: val = wxDateSpanPtr(val); val.thisown = 1
    return val

def wxDateSpan_Months(*_args, **_kwargs):
    val = apply(utilsc.wxDateSpan_Months,_args,_kwargs)
    if val: val = wxDateSpanPtr(val); val.thisown = 1
    return val

def wxDateSpan_Month(*_args, **_kwargs):
    val = apply(utilsc.wxDateSpan_Month,_args,_kwargs)
    if val: val = wxDateSpanPtr(val); val.thisown = 1
    return val

def wxDateSpan_Years(*_args, **_kwargs):
    val = apply(utilsc.wxDateSpan_Years,_args,_kwargs)
    if val: val = wxDateSpanPtr(val); val.thisown = 1
    return val

def wxDateSpan_Year(*_args, **_kwargs):
    val = apply(utilsc.wxDateSpan_Year,_args,_kwargs)
    if val: val = wxDateSpanPtr(val); val.thisown = 1
    return val



#-------------- VARIABLE WRAPPERS ------------------

wxCONFIG_USE_LOCAL_FILE = utilsc.wxCONFIG_USE_LOCAL_FILE
wxCONFIG_USE_GLOBAL_FILE = utilsc.wxCONFIG_USE_GLOBAL_FILE
wxCONFIG_USE_RELATIVE_PATH = utilsc.wxCONFIG_USE_RELATIVE_PATH
wxCONFIG_USE_NO_ESCAPE_CHARACTERS = utilsc.wxCONFIG_USE_NO_ESCAPE_CHARACTERS



# This file was created automatically by SWIG.
import miscc
class wxObjectPtr :
    def __init__(self,this):
        self.this = this
        self.thisown = 0
    def GetClassName(self, *_args, **_kwargs):
        val = apply(miscc.wxObject_GetClassName,(self,) + _args, _kwargs)
        return val
    def Destroy(self, *_args, **_kwargs):
        val = apply(miscc.wxObject_Destroy,(self,) + _args, _kwargs)
        return val
    def __repr__(self):
        return "<C wxObject instance at %s>" % (self.this,)
class wxObject(wxObjectPtr):
    def __init__(self,this):
        self.this = this




class wxSizePtr :
    def __init__(self,this):
        self.this = this
        self.thisown = 0
    def __del__(self, delfunc=miscc.delete_wxSize):
        if self.thisown == 1:
            try:
                delfunc(self)
            except:
                pass
    def Set(self, *_args, **_kwargs):
        val = apply(miscc.wxSize_Set,(self,) + _args, _kwargs)
        return val
    def GetX(self, *_args, **_kwargs):
        val = apply(miscc.wxSize_GetX,(self,) + _args, _kwargs)
        return val
    def GetY(self, *_args, **_kwargs):
        val = apply(miscc.wxSize_GetY,(self,) + _args, _kwargs)
        return val
    def GetWidth(self, *_args, **_kwargs):
        val = apply(miscc.wxSize_GetWidth,(self,) + _args, _kwargs)
        return val
    def GetHeight(self, *_args, **_kwargs):
        val = apply(miscc.wxSize_GetHeight,(self,) + _args, _kwargs)
        return val
    def SetWidth(self, *_args, **_kwargs):
        val = apply(miscc.wxSize_SetWidth,(self,) + _args, _kwargs)
        return val
    def SetHeight(self, *_args, **_kwargs):
        val = apply(miscc.wxSize_SetHeight,(self,) + _args, _kwargs)
        return val
    def asTuple(self, *_args, **_kwargs):
        val = apply(miscc.wxSize_asTuple,(self,) + _args, _kwargs)
        return val
    def __eq__(self, *_args, **_kwargs):
        val = apply(miscc.wxSize___eq__,(self,) + _args, _kwargs)
        return val
    def __ne__(self, *_args, **_kwargs):
        val = apply(miscc.wxSize___ne__,(self,) + _args, _kwargs)
        return val
    def __setattr__(self,name,value):
        if name == "x" :
            miscc.wxSize_x_set(self,value)
            return
        if name == "y" :
            miscc.wxSize_y_set(self,value)
            return
        if name == "width" :
            miscc.wxSize_width_set(self,value)
            return
        if name == "height" :
            miscc.wxSize_height_set(self,value)
            return
        self.__dict__[name] = value
    def __getattr__(self,name):
        if name == "x" : 
            return miscc.wxSize_x_get(self)
        if name == "y" : 
            return miscc.wxSize_y_get(self)
        if name == "width" : 
            return miscc.wxSize_width_get(self)
        if name == "height" : 
            return miscc.wxSize_height_get(self)
        raise AttributeError,name
    def __repr__(self):
        return "<C wxSize instance at %s>" % (self.this,)
    
    def __str__(self):                   return str(self.asTuple())
    def __repr__(self):                  return 'wxSize'+str(self.asTuple())
    def __len__(self):                   return len(self.asTuple())
    def __getitem__(self, index):        return self.asTuple()[index]
    def __setitem__(self, index, val):
        if index == 0: self.width = val
        elif index == 1: self.height = val
        else: raise IndexError
    def __nonzero__(self):               return self.asTuple() != (0,0)
    def __getinitargs__(self):           return ()
    def __getstate__(self):              return self.asTuple()
    def __setstate__(self, state):       self.Set(*state)

class wxSize(wxSizePtr):
    def __init__(self,*_args,**_kwargs):
        self.this = apply(miscc.new_wxSize,_args,_kwargs)
        self.thisown = 1




class wxRealPointPtr :
    def __init__(self,this):
        self.this = this
        self.thisown = 0
    def __del__(self, delfunc=miscc.delete_wxRealPoint):
        if self.thisown == 1:
            try:
                delfunc(self)
            except:
                pass
    def Set(self, *_args, **_kwargs):
        val = apply(miscc.wxRealPoint_Set,(self,) + _args, _kwargs)
        return val
    def asTuple(self, *_args, **_kwargs):
        val = apply(miscc.wxRealPoint_asTuple,(self,) + _args, _kwargs)
        return val
    def __add__(self, *_args, **_kwargs):
        val = apply(miscc.wxRealPoint___add__,(self,) + _args, _kwargs)
        if val: val = wxRealPointPtr(val) ; val.thisown = 1
        return val
    def __sub__(self, *_args, **_kwargs):
        val = apply(miscc.wxRealPoint___sub__,(self,) + _args, _kwargs)
        if val: val = wxRealPointPtr(val) ; val.thisown = 1
        return val
    def __eq__(self, *_args, **_kwargs):
        val = apply(miscc.wxRealPoint___eq__,(self,) + _args, _kwargs)
        return val
    def __ne__(self, *_args, **_kwargs):
        val = apply(miscc.wxRealPoint___ne__,(self,) + _args, _kwargs)
        return val
    def __setattr__(self,name,value):
        if name == "x" :
            miscc.wxRealPoint_x_set(self,value)
            return
        if name == "y" :
            miscc.wxRealPoint_y_set(self,value)
            return
        self.__dict__[name] = value
    def __getattr__(self,name):
        if name == "x" : 
            return miscc.wxRealPoint_x_get(self)
        if name == "y" : 
            return miscc.wxRealPoint_y_get(self)
        raise AttributeError,name
    def __repr__(self):
        return "<C wxRealPoint instance at %s>" % (self.this,)
    
    def __str__(self):                   return str(self.asTuple())
    def __repr__(self):                  return 'wxRealPoint'+str(self.asTuple())
    def __len__(self):                   return len(self.asTuple())
    def __getitem__(self, index):        return self.asTuple()[index]
    def __setitem__(self, index, val):
        if index == 0: self.width = val
        elif index == 1: self.height = val
        else: raise IndexError
    def __nonzero__(self):      return self.asTuple() != (0.0, 0.0)
    def __getinitargs__(self):           return ()
    def __getstate__(self):              return self.asTuple()
    def __setstate__(self, state):       self.Set(*state)

class wxRealPoint(wxRealPointPtr):
    def __init__(self,*_args,**_kwargs):
        self.this = apply(miscc.new_wxRealPoint,_args,_kwargs)
        self.thisown = 1




class wxPointPtr :
    def __init__(self,this):
        self.this = this
        self.thisown = 0
    def __del__(self, delfunc=miscc.delete_wxPoint):
        if self.thisown == 1:
            try:
                delfunc(self)
            except:
                pass
    def Set(self, *_args, **_kwargs):
        val = apply(miscc.wxPoint_Set,(self,) + _args, _kwargs)
        return val
    def asTuple(self, *_args, **_kwargs):
        val = apply(miscc.wxPoint_asTuple,(self,) + _args, _kwargs)
        return val
    def __add__(self, *_args, **_kwargs):
        val = apply(miscc.wxPoint___add__,(self,) + _args, _kwargs)
        if val: val = wxPointPtr(val) ; val.thisown = 1
        return val
    def __sub__(self, *_args, **_kwargs):
        val = apply(miscc.wxPoint___sub__,(self,) + _args, _kwargs)
        if val: val = wxPointPtr(val) ; val.thisown = 1
        return val
    def __eq__(self, *_args, **_kwargs):
        val = apply(miscc.wxPoint___eq__,(self,) + _args, _kwargs)
        return val
    def __ne__(self, *_args, **_kwargs):
        val = apply(miscc.wxPoint___ne__,(self,) + _args, _kwargs)
        return val
    def __setattr__(self,name,value):
        if name == "x" :
            miscc.wxPoint_x_set(self,value)
            return
        if name == "y" :
            miscc.wxPoint_y_set(self,value)
            return
        self.__dict__[name] = value
    def __getattr__(self,name):
        if name == "x" : 
            return miscc.wxPoint_x_get(self)
        if name == "y" : 
            return miscc.wxPoint_y_get(self)
        raise AttributeError,name
    def __repr__(self):
        return "<C wxPoint instance at %s>" % (self.this,)
    
    def __str__(self):                   return str(self.asTuple())
    def __repr__(self):                  return 'wxPoint'+str(self.asTuple())
    def __len__(self):                   return len(self.asTuple())
    def __getitem__(self, index):        return self.asTuple()[index]
    def __setitem__(self, index, val):
        if index == 0: self.x = val
        elif index == 1: self.y = val
        else: raise IndexError
    def __nonzero__(self):      return self.asTuple() != (0,0)
    def __getinitargs__(self):           return ()
    def __getstate__(self):              return self.asTuple()
    def __setstate__(self, state):       self.Set(*state)

class wxPoint(wxPointPtr):
    def __init__(self,*_args,**_kwargs):
        self.this = apply(miscc.new_wxPoint,_args,_kwargs)
        self.thisown = 1




class wxRectPtr :
    def __init__(self,this):
        self.this = this
        self.thisown = 0
    def __del__(self, delfunc=miscc.delete_wxRect):
        if self.thisown == 1:
            try:
                delfunc(self)
            except:
                pass
    def GetX(self, *_args, **_kwargs):
        val = apply(miscc.wxRect_GetX,(self,) + _args, _kwargs)
        return val
    def SetX(self, *_args, **_kwargs):
        val = apply(miscc.wxRect_SetX,(self,) + _args, _kwargs)
        return val
    def GetY(self, *_args, **_kwargs):
        val = apply(miscc.wxRect_GetY,(self,) + _args, _kwargs)
        return val
    def SetY(self, *_args, **_kwargs):
        val = apply(miscc.wxRect_SetY,(self,) + _args, _kwargs)
        return val
    def GetWidth(self, *_args, **_kwargs):
        val = apply(miscc.wxRect_GetWidth,(self,) + _args, _kwargs)
        return val
    def SetWidth(self, *_args, **_kwargs):
        val = apply(miscc.wxRect_SetWidth,(self,) + _args, _kwargs)
        return val
    def GetHeight(self, *_args, **_kwargs):
        val = apply(miscc.wxRect_GetHeight,(self,) + _args, _kwargs)
        return val
    def SetHeight(self, *_args, **_kwargs):
        val = apply(miscc.wxRect_SetHeight,(self,) + _args, _kwargs)
        return val
    def GetPosition(self, *_args, **_kwargs):
        val = apply(miscc.wxRect_GetPosition,(self,) + _args, _kwargs)
        if val: val = wxPointPtr(val) ; val.thisown = 1
        return val
    def GetSize(self, *_args, **_kwargs):
        val = apply(miscc.wxRect_GetSize,(self,) + _args, _kwargs)
        if val: val = wxSizePtr(val) ; val.thisown = 1
        return val
    def SetPosition(self, *_args, **_kwargs):
        val = apply(miscc.wxRect_SetPosition,(self,) + _args, _kwargs)
        return val
    def SetSize(self, *_args, **_kwargs):
        val = apply(miscc.wxRect_SetSize,(self,) + _args, _kwargs)
        return val
    def GetLeft(self, *_args, **_kwargs):
        val = apply(miscc.wxRect_GetLeft,(self,) + _args, _kwargs)
        return val
    def GetTop(self, *_args, **_kwargs):
        val = apply(miscc.wxRect_GetTop,(self,) + _args, _kwargs)
        return val
    def GetBottom(self, *_args, **_kwargs):
        val = apply(miscc.wxRect_GetBottom,(self,) + _args, _kwargs)
        return val
    def GetRight(self, *_args, **_kwargs):
        val = apply(miscc.wxRect_GetRight,(self,) + _args, _kwargs)
        return val
    def SetLeft(self, *_args, **_kwargs):
        val = apply(miscc.wxRect_SetLeft,(self,) + _args, _kwargs)
        return val
    def SetRight(self, *_args, **_kwargs):
        val = apply(miscc.wxRect_SetRight,(self,) + _args, _kwargs)
        return val
    def SetTop(self, *_args, **_kwargs):
        val = apply(miscc.wxRect_SetTop,(self,) + _args, _kwargs)
        return val
    def SetBottom(self, *_args, **_kwargs):
        val = apply(miscc.wxRect_SetBottom,(self,) + _args, _kwargs)
        return val
    def Deflate(self, *_args, **_kwargs):
        val = apply(miscc.wxRect_Deflate,(self,) + _args, _kwargs)
        return val
    def Inflate(self, *_args, **_kwargs):
        val = apply(miscc.wxRect_Inflate,(self,) + _args, _kwargs)
        return val
    def InsideXY(self, *_args, **_kwargs):
        val = apply(miscc.wxRect_InsideXY,(self,) + _args, _kwargs)
        return val
    def Inside(self, *_args, **_kwargs):
        val = apply(miscc.wxRect_Inside,(self,) + _args, _kwargs)
        return val
    def Intersects(self, *_args, **_kwargs):
        val = apply(miscc.wxRect_Intersects,(self,) + _args, _kwargs)
        return val
    def OffsetXY(self, *_args, **_kwargs):
        val = apply(miscc.wxRect_OffsetXY,(self,) + _args, _kwargs)
        return val
    def Offset(self, *_args, **_kwargs):
        val = apply(miscc.wxRect_Offset,(self,) + _args, _kwargs)
        return val
    def Set(self, *_args, **_kwargs):
        val = apply(miscc.wxRect_Set,(self,) + _args, _kwargs)
        return val
    def asTuple(self, *_args, **_kwargs):
        val = apply(miscc.wxRect_asTuple,(self,) + _args, _kwargs)
        return val
    def __add__(self, *_args, **_kwargs):
        val = apply(miscc.wxRect___add__,(self,) + _args, _kwargs)
        if val: val = wxRectPtr(val) ; val.thisown = 1
        return val
    def __eq__(self, *_args, **_kwargs):
        val = apply(miscc.wxRect___eq__,(self,) + _args, _kwargs)
        return val
    def __ne__(self, *_args, **_kwargs):
        val = apply(miscc.wxRect___ne__,(self,) + _args, _kwargs)
        return val
    def __setattr__(self,name,value):
        if name == "x" :
            miscc.wxRect_x_set(self,value)
            return
        if name == "y" :
            miscc.wxRect_y_set(self,value)
            return
        if name == "width" :
            miscc.wxRect_width_set(self,value)
            return
        if name == "height" :
            miscc.wxRect_height_set(self,value)
            return
        self.__dict__[name] = value
    def __getattr__(self,name):
        if name == "x" : 
            return miscc.wxRect_x_get(self)
        if name == "y" : 
            return miscc.wxRect_y_get(self)
        if name == "width" : 
            return miscc.wxRect_width_get(self)
        if name == "height" : 
            return miscc.wxRect_height_get(self)
        raise AttributeError,name
    def __repr__(self):
        return "<C wxRect instance at %s>" % (self.this,)
    
    def __str__(self):                   return str(self.asTuple())
    def __repr__(self):                  return 'wxRect'+str(self.asTuple())
    def __len__(self):                   return len(self.asTuple())
    def __getitem__(self, index):        return self.asTuple()[index]
    def __setitem__(self, index, val):
        if index == 0: self.x = val
        elif index == 1: self.y = val
        elif index == 2: self.width = val
        elif index == 3: self.height = val
        else: raise IndexError
    def __nonzero__(self):               return self.asTuple() != (0,0,0,0)
    def __getinitargs__(self):           return ()
    def __getstate__(self):              return self.asTuple()
    def __setstate__(self, state):       self.Set(*state)

    # override the __getattr__ made by SWIG
    def __getattr__(self, name):
        d = {
            'x' : miscc.wxRect_x_get,
            'y' : miscc.wxRect_y_get,
            'width' : miscc.wxRect_width_get,
            'height' : miscc.wxRect_height_get,
            'top' : miscc.wxRect_GetTop,
            'bottom' : miscc.wxRect_GetBottom,
            'left' : miscc.wxRect_GetLeft,
            'right' : miscc.wxRect_GetRight,
            }
        try:
            func = d[name]
        except KeyError:
            raise AttributeError,name
        return func(self)

    # and also the __setattr__
    def __setattr__(self, name, value):
        d = {
            'x' : miscc.wxRect_x_set,
            'y' : miscc.wxRect_y_set,
            'width' : miscc.wxRect_width_set,
            'height' : miscc.wxRect_height_set,
            'top' : miscc.wxRect_SetTop,
            'bottom' : miscc.wxRect_SetBottom,
            'left' : miscc.wxRect_SetLeft,
            'right' : miscc.wxRect_SetRight,
            }
        try:
            func = d[name]
        except KeyError:
            self.__dict__[name] = value
            return
        func(self, value)
    
class wxRect(wxRectPtr):
    def __init__(self,*_args,**_kwargs):
        self.this = apply(miscc.new_wxRect,_args,_kwargs)
        self.thisown = 1




class wxPoint2DDoublePtr :
    def __init__(self,this):
        self.this = this
        self.thisown = 0
    def GetFloor(self, *_args, **_kwargs):
        val = apply(miscc.wxPoint2DDouble_GetFloor,(self,) + _args, _kwargs)
        return val
    def GetRounded(self, *_args, **_kwargs):
        val = apply(miscc.wxPoint2DDouble_GetRounded,(self,) + _args, _kwargs)
        return val
    def GetVectorLength(self, *_args, **_kwargs):
        val = apply(miscc.wxPoint2DDouble_GetVectorLength,(self,) + _args, _kwargs)
        return val
    def GetVectorAngle(self, *_args, **_kwargs):
        val = apply(miscc.wxPoint2DDouble_GetVectorAngle,(self,) + _args, _kwargs)
        return val
    def SetVectorLength(self, *_args, **_kwargs):
        val = apply(miscc.wxPoint2DDouble_SetVectorLength,(self,) + _args, _kwargs)
        return val
    def SetVectorAngle(self, *_args, **_kwargs):
        val = apply(miscc.wxPoint2DDouble_SetVectorAngle,(self,) + _args, _kwargs)
        return val
    def GetDistance(self, *_args, **_kwargs):
        val = apply(miscc.wxPoint2DDouble_GetDistance,(self,) + _args, _kwargs)
        return val
    def GetDistanceSquare(self, *_args, **_kwargs):
        val = apply(miscc.wxPoint2DDouble_GetDistanceSquare,(self,) + _args, _kwargs)
        return val
    def GetDotProduct(self, *_args, **_kwargs):
        val = apply(miscc.wxPoint2DDouble_GetDotProduct,(self,) + _args, _kwargs)
        return val
    def GetCrossProduct(self, *_args, **_kwargs):
        val = apply(miscc.wxPoint2DDouble_GetCrossProduct,(self,) + _args, _kwargs)
        return val
    def Set(self, *_args, **_kwargs):
        val = apply(miscc.wxPoint2DDouble_Set,(self,) + _args, _kwargs)
        return val
    def __neg__(self, *_args, **_kwargs):
        val = apply(miscc.wxPoint2DDouble___neg__,(self,) + _args, _kwargs)
        if val: val = wxPoint2DDoublePtr(val) ; val.thisown = 1
        return val
    def __iadd__(self, *_args, **_kwargs):
        val = apply(miscc.wxPoint2DDouble___iadd__,(self,) + _args, _kwargs)
        if val: val = wxPoint2DDoublePtr(val) 
        return val
    def __isub__(self, *_args, **_kwargs):
        val = apply(miscc.wxPoint2DDouble___isub__,(self,) + _args, _kwargs)
        if val: val = wxPoint2DDoublePtr(val) 
        return val
    def __imul__(self, *_args, **_kwargs):
        val = apply(miscc.wxPoint2DDouble___imul__,(self,) + _args, _kwargs)
        if val: val = wxPoint2DDoublePtr(val) 
        return val
    def __idiv__(self, *_args, **_kwargs):
        val = apply(miscc.wxPoint2DDouble___idiv__,(self,) + _args, _kwargs)
        if val: val = wxPoint2DDoublePtr(val) 
        return val
    def __eq__(self, *_args, **_kwargs):
        val = apply(miscc.wxPoint2DDouble___eq__,(self,) + _args, _kwargs)
        return val
    def __ne__(self, *_args, **_kwargs):
        val = apply(miscc.wxPoint2DDouble___ne__,(self,) + _args, _kwargs)
        return val
    def asTuple(self, *_args, **_kwargs):
        val = apply(miscc.wxPoint2DDouble_asTuple,(self,) + _args, _kwargs)
        return val
    def __setattr__(self,name,value):
        if name == "m_x" :
            miscc.wxPoint2DDouble_m_x_set(self,value)
            return
        if name == "m_y" :
            miscc.wxPoint2DDouble_m_y_set(self,value)
            return
        if name == "x" :
            miscc.wxPoint2DDouble_x_set(self,value)
            return
        if name == "y" :
            miscc.wxPoint2DDouble_y_set(self,value)
            return
        self.__dict__[name] = value
    def __getattr__(self,name):
        if name == "m_x" : 
            return miscc.wxPoint2DDouble_m_x_get(self)
        if name == "m_y" : 
            return miscc.wxPoint2DDouble_m_y_get(self)
        if name == "x" : 
            return miscc.wxPoint2DDouble_x_get(self)
        if name == "y" : 
            return miscc.wxPoint2DDouble_y_get(self)
        raise AttributeError,name
    def __repr__(self):
        return "<C wxPoint2DDouble instance at %s>" % (self.this,)
    
    def SetPolarCoordinates(self, angle, length):
        self.SetVectorLength(length)
        self.SetVectorAngle(angle)
    def Normalize(self):
        self.SetVectorLength(1.0)
    
    
    def __str__(self):                   return str(self.asTuple())
    def __repr__(self):                  return 'wxPoint2DDouble'+str(self.asTuple())
    def __len__(self):                   return len(self.asTuple())
    def __getitem__(self, index):        return self.asTuple()[index]
    def __setitem__(self, index, val):
        if index == 0: self.m_x = val
        elif index == 1: self.m_yt = val
        else: raise IndexError
    def __nonzero__(self):               return self.asTuple() != (0.0, 0.0)
    def __getinitargs__(self):           return ()
    def __getstate__(self):              return self.asTuple()
    def __setstate__(self, state):       self.Set(*state)

class wxPoint2DDouble(wxPoint2DDoublePtr):
    def __init__(self,*_args,**_kwargs):
        self.this = apply(miscc.new_wxPoint2DDouble,_args,_kwargs)
        self.thisown = 1



def wxPoint2DDoubleCopy(*_args,**_kwargs):
    val = wxPoint2DDoublePtr(apply(miscc.new_wxPoint2DDoubleCopy,_args,_kwargs))
    val.thisown = 1
    return val

def wxPoint2DDoubleFromPoint(*_args,**_kwargs):
    val = wxPoint2DDoublePtr(apply(miscc.new_wxPoint2DDoubleFromPoint,_args,_kwargs))
    val.thisown = 1
    return val


class wxIndividualLayoutConstraintPtr(wxObjectPtr):
    def __init__(self,this):
        self.this = this
        self.thisown = 0
    def Above(self, *_args, **_kwargs):
        val = apply(miscc.wxIndividualLayoutConstraint_Above,(self,) + _args, _kwargs)
        return val
    def Absolute(self, *_args, **_kwargs):
        val = apply(miscc.wxIndividualLayoutConstraint_Absolute,(self,) + _args, _kwargs)
        return val
    def AsIs(self, *_args, **_kwargs):
        val = apply(miscc.wxIndividualLayoutConstraint_AsIs,(self,) + _args, _kwargs)
        return val
    def Below(self, *_args, **_kwargs):
        val = apply(miscc.wxIndividualLayoutConstraint_Below,(self,) + _args, _kwargs)
        return val
    def Unconstrained(self, *_args, **_kwargs):
        val = apply(miscc.wxIndividualLayoutConstraint_Unconstrained,(self,) + _args, _kwargs)
        return val
    def LeftOf(self, *_args, **_kwargs):
        val = apply(miscc.wxIndividualLayoutConstraint_LeftOf,(self,) + _args, _kwargs)
        return val
    def PercentOf(self, *_args, **_kwargs):
        val = apply(miscc.wxIndividualLayoutConstraint_PercentOf,(self,) + _args, _kwargs)
        return val
    def RightOf(self, *_args, **_kwargs):
        val = apply(miscc.wxIndividualLayoutConstraint_RightOf,(self,) + _args, _kwargs)
        return val
    def SameAs(self, *_args, **_kwargs):
        val = apply(miscc.wxIndividualLayoutConstraint_SameAs,(self,) + _args, _kwargs)
        return val
    def Set(self, *_args, **_kwargs):
        val = apply(miscc.wxIndividualLayoutConstraint_Set,(self,) + _args, _kwargs)
        return val
    def __repr__(self):
        return "<C wxIndividualLayoutConstraint instance at %s>" % (self.this,)
class wxIndividualLayoutConstraint(wxIndividualLayoutConstraintPtr):
    def __init__(self,this):
        self.this = this




class wxLayoutConstraintsPtr(wxObjectPtr):
    def __init__(self,this):
        self.this = this
        self.thisown = 0
    def __setattr__(self,name,value):
        if name == "bottom" :
            miscc.wxLayoutConstraints_bottom_set(self,value.this)
            return
        if name == "centreX" :
            miscc.wxLayoutConstraints_centreX_set(self,value.this)
            return
        if name == "centreY" :
            miscc.wxLayoutConstraints_centreY_set(self,value.this)
            return
        if name == "height" :
            miscc.wxLayoutConstraints_height_set(self,value.this)
            return
        if name == "left" :
            miscc.wxLayoutConstraints_left_set(self,value.this)
            return
        if name == "right" :
            miscc.wxLayoutConstraints_right_set(self,value.this)
            return
        if name == "top" :
            miscc.wxLayoutConstraints_top_set(self,value.this)
            return
        if name == "width" :
            miscc.wxLayoutConstraints_width_set(self,value.this)
            return
        self.__dict__[name] = value
    def __getattr__(self,name):
        if name == "bottom" : 
            return wxIndividualLayoutConstraintPtr(miscc.wxLayoutConstraints_bottom_get(self))
        if name == "centreX" : 
            return wxIndividualLayoutConstraintPtr(miscc.wxLayoutConstraints_centreX_get(self))
        if name == "centreY" : 
            return wxIndividualLayoutConstraintPtr(miscc.wxLayoutConstraints_centreY_get(self))
        if name == "height" : 
            return wxIndividualLayoutConstraintPtr(miscc.wxLayoutConstraints_height_get(self))
        if name == "left" : 
            return wxIndividualLayoutConstraintPtr(miscc.wxLayoutConstraints_left_get(self))
        if name == "right" : 
            return wxIndividualLayoutConstraintPtr(miscc.wxLayoutConstraints_right_get(self))
        if name == "top" : 
            return wxIndividualLayoutConstraintPtr(miscc.wxLayoutConstraints_top_get(self))
        if name == "width" : 
            return wxIndividualLayoutConstraintPtr(miscc.wxLayoutConstraints_width_get(self))
        raise AttributeError,name
    def __repr__(self):
        return "<C wxLayoutConstraints instance at %s>" % (self.this,)
class wxLayoutConstraints(wxLayoutConstraintsPtr):
    def __init__(self,*_args,**_kwargs):
        self.this = apply(miscc.new_wxLayoutConstraints,_args,_kwargs)
        self.thisown = 1




class wxAcceleratorEntryPtr :
    def __init__(self,this):
        self.this = this
        self.thisown = 0
    def __del__(self, delfunc=miscc.delete_wxAcceleratorEntry):
        if self.thisown == 1:
            try:
                delfunc(self)
            except:
                pass
    def Set(self, *_args, **_kwargs):
        val = apply(miscc.wxAcceleratorEntry_Set,(self,) + _args, _kwargs)
        return val
    def GetFlags(self, *_args, **_kwargs):
        val = apply(miscc.wxAcceleratorEntry_GetFlags,(self,) + _args, _kwargs)
        return val
    def GetKeyCode(self, *_args, **_kwargs):
        val = apply(miscc.wxAcceleratorEntry_GetKeyCode,(self,) + _args, _kwargs)
        return val
    def GetCommand(self, *_args, **_kwargs):
        val = apply(miscc.wxAcceleratorEntry_GetCommand,(self,) + _args, _kwargs)
        return val
    def __repr__(self):
        return "<C wxAcceleratorEntry instance at %s>" % (self.this,)
class wxAcceleratorEntry(wxAcceleratorEntryPtr):
    def __init__(self,*_args,**_kwargs):
        self.this = apply(miscc.new_wxAcceleratorEntry,_args,_kwargs)
        self.thisown = 1




class wxAcceleratorTablePtr(wxObjectPtr):
    def __init__(self,this):
        self.this = this
        self.thisown = 0
    def __del__(self, delfunc=miscc.delete_wxAcceleratorTable):
        if self.thisown == 1:
            try:
                delfunc(self)
            except:
                pass
    def __repr__(self):
        return "<C wxAcceleratorTable instance at %s>" % (self.this,)
class wxAcceleratorTable(wxAcceleratorTablePtr):
    def __init__(self,*_args,**_kwargs):
        self.this = apply(miscc.new_wxAcceleratorTable,_args,_kwargs)
        self.thisown = 1




class wxBusyInfoPtr(wxObjectPtr):
    def __init__(self,this):
        self.this = this
        self.thisown = 0
    def __del__(self, delfunc=miscc.delete_wxBusyInfo):
        if self.thisown == 1:
            try:
                delfunc(self)
            except:
                pass
    def __repr__(self):
        return "<C wxBusyInfo instance at %s>" % (self.this,)
class wxBusyInfo(wxBusyInfoPtr):
    def __init__(self,*_args,**_kwargs):
        self.this = apply(miscc.new_wxBusyInfo,_args,_kwargs)
        self.thisown = 1






#-------------- FUNCTION WRAPPERS ------------------

wxIntersectRect = miscc.wxIntersectRect

wxNewId = miscc.wxNewId

wxRegisterId = miscc.wxRegisterId

wxGetCurrentId = miscc.wxGetCurrentId

wxBell = miscc.wxBell

wxEndBusyCursor = miscc.wxEndBusyCursor

wxGetElapsedTime = miscc.wxGetElapsedTime

wxGetMousePosition = miscc.wxGetMousePosition

wxIsBusy = miscc.wxIsBusy

wxNow = miscc.wxNow

wxShell = miscc.wxShell

wxStartTimer = miscc.wxStartTimer

wxGetOsVersion = miscc.wxGetOsVersion

wxGetOsDescription = miscc.wxGetOsDescription

wxShutdown = miscc.wxShutdown

wxSleep = miscc.wxSleep

wxUsleep = miscc.wxUsleep

wxYield = miscc.wxYield

wxYieldIfNeeded = miscc.wxYieldIfNeeded

wxEnableTopLevelWindows = miscc.wxEnableTopLevelWindows

wxStripMenuCodes = miscc.wxStripMenuCodes

wxGetEmailAddress = miscc.wxGetEmailAddress

wxGetHostName = miscc.wxGetHostName

wxGetFullHostName = miscc.wxGetFullHostName

wxGetUserId = miscc.wxGetUserId

wxGetUserName = miscc.wxGetUserName

wxGetHomeDir = miscc.wxGetHomeDir

wxGetUserHome = miscc.wxGetUserHome

wxGetProcessId = miscc.wxGetProcessId

def wxGetAccelFromString(*_args, **_kwargs):
    val = apply(miscc.wxGetAccelFromString,_args,_kwargs)
    if val: val = wxAcceleratorEntryPtr(val)
    return val



#-------------- VARIABLE WRAPPERS ------------------

wxSHUTDOWN_POWEROFF = miscc.wxSHUTDOWN_POWEROFF
wxSHUTDOWN_REBOOT = miscc.wxSHUTDOWN_REBOOT
wxLeft = miscc.wxLeft
wxTop = miscc.wxTop
wxRight = miscc.wxRight
wxBottom = miscc.wxBottom
wxWidth = miscc.wxWidth
wxHeight = miscc.wxHeight
wxCentre = miscc.wxCentre
wxCenter = miscc.wxCenter
wxCentreX = miscc.wxCentreX
wxCentreY = miscc.wxCentreY
wxUnconstrained = miscc.wxUnconstrained
wxAsIs = miscc.wxAsIs
wxPercentOf = miscc.wxPercentOf
wxAbove = miscc.wxAbove
wxBelow = miscc.wxBelow
wxLeftOf = miscc.wxLeftOf
wxRightOf = miscc.wxRightOf
wxSameAs = miscc.wxSameAs
wxAbsolute = miscc.wxAbsolute
cvar = miscc.cvar
wxNullAcceleratorTable = wxAcceleratorTablePtr(miscc.cvar.wxNullAcceleratorTable)



"""Decorator classes for documentation and shell scripting.
"""

__author__ = "Patrick K. O'Brien <pobrien@orbtech.com>"
__cvsid__ = "$Id: ImageHandlers.py,v 1.1.2.1 2003/03/11 19:37:12 RD Exp $"
__revision__ = "$Revision: 1.1.2.1 $"[11:-2]


# These are not the real wxPython classes. These are Python versions
# for documentation purposes. They are also used to apply docstrings
# to the real wxPython classes, which are SWIG-generated wrappers for
# C-language classes.


from Base import Object
import Parameters as wx


class ImageHandler(Object):
    """"""

    def CanRead(self):
        """"""
        pass

    def GetExtension(self):
        """"""
        pass

    def GetMimeType(self):
        """"""
        pass

    def GetName(self):
        """"""
        pass

    def GetType(self):
        """"""
        pass

    def SetExtension(self):
        """"""
        pass

    def SetMimeType(self):
        """"""
        pass

    def SetName(self):
        """"""
        pass

    def SetType(self):
        """"""
        pass

    def __init__(self):
        """"""
        pass


class BMPHandler(ImageHandler):
    """"""

    def __init__(self):
        """"""
        pass


class GIFHandler(ImageHandler):
    """"""

    def __init__(self):
        """"""
        pass


class ICOHandler(BMPHandler):
    """"""

    def __init__(self):
        """"""
        pass


class CURHandler(ICOHandler):
    """"""

    def __init__(self):
        """"""
        pass


class ANIHandler(CURHandler):
    """"""

    def __init__(self):
        """"""
        pass


class JPEGHandler(ImageHandler):
    """"""

    def __init__(self):
        """"""
        pass


class PCXHandler(ImageHandler):
    """"""

    def __init__(self):
        """"""
        pass


class PNGHandler(ImageHandler):
    """"""

    def __init__(self):
        """"""
        pass


class PNMHandler(ImageHandler):
    """"""

    def __init__(self):
        """"""
        pass


class TIFFHandler(ImageHandler):
    """"""

    def __init__(self):
        """"""
        pass





"""Decorator classes for documentation and shell scripting.
"""

__author__ = "Patrick K. O'Brien <pobrien@orbtech.com>"
__cvsid__ = "$Id: DataStructures.py,v 1.1.2.1 2003/03/11 19:37:13 RD Exp $"
__revision__ = "$Revision: 1.1.2.1 $"[11:-2]


# These are not the real wxPython classes. These are Python versions
# for documentation purposes. They are also used to apply docstrings
# to the real wxPython classes, which are SWIG-generated wrappers for
# C-language classes.


import Parameters as wx


class Point:
    """"""

    def Set(self):
        """"""
        pass

    def __add__(self):
        """"""
        pass

    def __del__(self):
        """"""
        pass

    def __eq__(self):
        """"""
        pass

    def __getattr__(self):
        """"""
        pass

    def __getitem__(self):
        """"""
        pass

    def __init__(self):
        """"""
        pass

    def __len__(self):
        """"""
        pass

    def __ne__(self):
        """"""
        pass

    def __nonzero__(self):
        """"""
        pass

    def __setattr__(self):
        """"""
        pass

    def __setitem__(self):
        """"""
        pass

    def __str__(self):
        """"""
        pass

    def __sub__(self):
        """"""
        pass

    def asTuple(self):
        """"""
        pass


class Point2DDouble:
    """"""

    def GetCrossProduct(self):
        """"""
        pass

    def GetDistance(self):
        """"""
        pass

    def GetDistanceSquare(self):
        """"""
        pass

    def GetDotProduct(self):
        """"""
        pass

    def GetFloor(self):
        """"""
        pass

    def GetRounded(self):
        """"""
        pass

    def GetVectorAngle(self):
        """"""
        pass

    def GetVectorLength(self):
        """"""
        pass

    def Normalize(self):
        """"""
        pass

    def SetPolarCoordinates(self):
        """"""
        pass

    def SetVectorAngle(self):
        """"""
        pass

    def SetVectorLength(self):
        """"""
        pass

    def __eq__(self):
        """"""
        pass

    def __getattr__(self):
        """"""
        pass

    def __getitem__(self):
        """"""
        pass

    def __iadd__(self):
        """"""
        pass

    def __idiv__(self):
        """"""
        pass

    def __imul__(self):
        """"""
        pass

    def __init__(self):
        """"""
        pass

    def __isub__(self):
        """"""
        pass

    def __len__(self):
        """"""
        pass

    def __ne__(self):
        """"""
        pass

    def __neg__(self):
        """"""
        pass

    def __nonzero__(self):
        """"""
        pass

    def __setattr__(self):
        """"""
        pass

    def __setitem__(self):
        """"""
        pass

    def __str__(self):
        """"""
        pass

    def asTuple(self):
        """"""
        pass


class RealPoint:
    """"""

    def Set(self):
        """"""
        pass

    def __add__(self):
        """"""
        pass

    def __del__(self):
        """"""
        pass

    def __eq__(self):
        """"""
        pass

    def __getattr__(self):
        """"""
        pass

    def __getitem__(self):
        """"""
        pass

    def __init__(self):
        """"""
        pass

    def __len__(self):
        """"""
        pass

    def __ne__(self):
        """"""
        pass

    def __nonzero__(self):
        """"""
        pass

    def __setattr__(self):
        """"""
        pass

    def __setitem__(self):
        """"""
        pass

    def __str__(self):
        """"""
        pass

    def __sub__(self):
        """"""
        pass

    def asTuple(self):
        """"""
        pass


class Rect:
    """"""

    def GetBottom(self):
        """"""
        pass

    def GetHeight(self):
        """"""
        pass

    def GetLeft(self):
        """"""
        pass

    def GetPosition(self):
        """"""
        pass

    def GetRight(self):
        """"""
        pass

    def GetSize(self):
        """"""
        pass

    def GetTop(self):
        """"""
        pass

    def GetWidth(self):
        """"""
        pass

    def GetX(self):
        """"""
        pass

    def GetY(self):
        """"""
        pass

    def Inflate(self):
        """"""
        pass

    def Inside(self):
        """"""
        pass

    def SetBottom(self):
        """"""
        pass

    def SetHeight(self):
        """"""
        pass

    def SetLeft(self):
        """"""
        pass

    def SetPosition(self):
        """"""
        pass

    def SetRight(self):
        """"""
        pass

    def SetSize(self):
        """"""
        pass

    def SetTop(self):
        """"""
        pass

    def SetWidth(self):
        """"""
        pass

    def SetX(self):
        """"""
        pass

    def SetY(self):
        """"""
        pass

    def __add__(self):
        """"""
        pass

    def __del__(self):
        """"""
        pass

    def __eq__(self):
        """"""
        pass

    def __getattr__(self):
        """"""
        pass

    def __getitem__(self):
        """"""
        pass

    def __init__(self):
        """"""
        pass

    def __len__(self):
        """"""
        pass

    def __ne__(self):
        """"""
        pass

    def __nonzero__(self):
        """"""
        pass

    def __setattr__(self):
        """"""
        pass

    def __setitem__(self):
        """"""
        pass

    def __str__(self):
        """"""
        pass

    def asTuple(self):
        """"""
        pass


class Size:
    """"""

    def GetHeight(self):
        """"""
        pass

    def GetWidth(self):
        """"""
        pass

    def GetX(self):
        """"""
        pass

    def GetY(self):
        """"""
        pass

    def Set(self):
        """"""
        pass

    def SetHeight(self):
        """"""
        pass

    def SetWidth(self):
        """"""
        pass

    def __del__(self):
        """"""
        pass

    def __eq__(self):
        """"""
        pass

    def __getattr__(self):
        """"""
        pass

    def __getitem__(self):
        """"""
        pass

    def __init__(self):
        """"""
        pass

    def __len__(self):
        """"""
        pass

    def __ne__(self):
        """"""
        pass

    def __nonzero__(self):
        """"""
        pass

    def __setattr__(self):
        """"""
        pass

    def __setitem__(self):
        """"""
        pass

    def __str__(self):
        """"""
        pass

    def asTuple(self):
        """"""
        pass





"""Decorator classes for documentation and shell scripting.
"""

__author__ = "Patrick K. O'Brien <pobrien@orbtech.com>"
__cvsid__ = "$Id: Base.py,v 1.1.2.2 2003/03/20 23:14:27 RD Exp $"
__revision__ = "$Revision: 1.1.2.2 $"[11:-2]


# These are not the real wxPython classes. These are Python versions
# for documentation purposes. They are also used to apply docstrings
# to the real wxPython classes, which are SWIG-generated wrappers for
# C-language classes.


import Parameters as wx


class Object:
    """Base class for all other wxPython classes."""
    
    def __init__(self):
        """Create a Object instance."""
        pass

    def Destroy(self):
        """Destroy the Object instance."""
        pass

    def GetClassName(self):
        """Return the name of the class."""
        pass


class EvtHandler(Object):
    """Base class that can handle events from the windowing system.

    If the handler is part of a chain, the destructor will unlink
    itself and restore the previous and next handlers so that they
    point to each other."""

    def __init__(self):
        """Create a EvtHandler instance."""
        pass

    def AddPendingEvent(self, event):
        """Post an event to be processed later.

        event is an Event instance to add to process queue.

        The difference between sending an event (using the
        ProcessEvent method) and posting it is that in the first case
        the event is processed before the function returns, while in
        the second case, the function returns immediately and the
        event will be processed sometime later (usually during the
        next event loop iteration).

        A copy of event is made by the function, so the original can
        be deleted as soon as function returns (it is common that the
        original is created on the stack).  This requires that the
        Event::Clone method be implemented by event so that it can
        be duplicated and stored until it gets processed.

        This is also the method to call for inter-thread
        communication.  It will post events safely between different
        threads which means that this method is thread-safe by using
        critical sections where needed.  In a multi-threaded program,
        you often need to inform the main GUI thread about the status
        of other working threads and such notification should be done
        using this method.

        This method automatically wakes up idle handling if the
        underlying window system is currently idle and thus would not
        send any idle events.  (Waking up idle handling is done
        calling WakeUpIdle.)"""
        pass

    def Connect(self, id, lastId, eventType, func):
        """Connects the given function dynamically with the event
        handler, id and event type.  This is an alternative to the use
        of static event tables.

        id is the identifier (or first of the identifier range) to be
        associated with the event handler function.

        lastId is the second part of the identifier range to be
        associated with the event handler function.

        eventType is the event type to be associated with this event
        handler.

        function is the event handler function.

        userData is data to be associated with the event table entry."""
        pass

    def Disconnect(self, id, lastId=-1, eventType=wx.EVT_NULL):
        """Disconnects the given function dynamically from the event
        handler, using the specified parameters as search criteria and
        returning True if a matching function has been found and
        removed.  This method can only disconnect functions which have
        been added using the EvtHandler.Connect method. There is no
        way to disconnect functions connected using the (static) event
        tables.

        id is the identifier (or first of the identifier range) to be
        associated with the event handler function.

        lastId is the second part of the identifier range to be
        associated with the event handler function.

        eventType is the event type to be associated with this event
        handler.

        function is the event handler function.

        userData is data to be associated with the event table entry."""
        pass

    def GetEvtHandlerEnabled(self):
        """Return True if the event handler is enabled, False
        otherwise."""
        pass

    def GetNextHandler(self):
        """Return the next handler in the chain."""
        pass

    def GetPreviousHandler(self):
        """Return the previous handler in the chain."""
        pass

    def ProcessEvent(self, event):
        """Processes an event, searching event tables and calling zero
        or more suitable event handler function(s).  Return True if a
        suitable event handler function was found and executed, and
        the function did not call Event.Skip().

        event is an Event to process.

        Normally, your application would not call this function: it is
        called in the wxPython implementation to dispatch incoming
        user interface events to the framework (and application).

        However, you might need to call it if implementing new
        functionality (such as a new control) where you define new
        event types, as opposed to allowing the user to override
        virtual functions.

        An instance where you might actually override the ProcessEvent
        function is where you want to direct event processing to event
        handlers not normally noticed by wxWindows.  For example, in
        the document/view architecture, documents and views are
        potential event handlers.  When an event reaches a frame,
        ProcessEvent will need to be called on the associated document
        and view in case event handler functions are associated with
        these objects.  The property classes library (Property) also
        overrides ProcessEvent for similar reasons.

        The normal order of event table searching is as follows:

        1. If the object is disabled (via a call to
        EvtHandler.SetEvtHandlerEnabled) the function skips to step
        (6).

        2. If the object is a Window, ProcessEvent is recursively
        called on the window's Validator.  If this returns TRUE, the
        function exits.

        3. SearchEventTable is called for this event handler.  If this
        fails, the base class table is tried, and so on until no more
        tables exist or an appropriate function was found, in which
        case the function exits.

        4. The search is applied down the entire chain of event
        handlers (usually the chain has a length of one).  If this
        succeeds, the function exits.

        5. If the object is a Window and the event is a
        CommandEvent, ProcessEvent is recursively applied to the
        parent window's event handler.  If this returns TRUE, the
        function exits.

        6. Finally, ProcessEvent is called on the App object.

        See also:

        EvtHandler::SearchEventTable"""
        pass

    def SetEvtHandlerEnabled(self, enabled):
        """Enable or disable the event handler.

        You can use this function to avoid having to remove the event
        handler from the chain, for example when implementing a dialog
        editor and changing from edit to test mode."""
        pass

    def SetNextHandler(self, handler):
        """Set the pointer to the next handler."""
        pass

    def SetPreviousHandler(self, handler):
        """Set the pointer to the previous handler."""
        pass





"""PyCrust Crust combines the shell and filling into one control."""

__author__ = "Patrick K. O'Brien <pobrien@orbtech.com>"
__cvsid__ = "$Id: crust.py,v 1.11.2.5 2003/03/20 23:14:32 RD Exp $"
__revision__ = "$Revision: 1.11.2.5 $"[11:-2]

from wxPython import wx
from filling import Filling
import os
from shell import Shell
from shellmenu import ShellMenu
from version import VERSION

try:
    True
except NameError:
    True = 1==1
    False = 1==0


class Crust(wx.wxSplitterWindow):
    """PyCrust Crust based on wxSplitterWindow."""

    name = 'PyCrust Crust'
    revision = __revision__

    def __init__(self, parent, id=-1, pos=wx.wxDefaultPosition, 
                 size=wx.wxDefaultSize, style=wx.wxSP_3D,
                 name='Crust Window', rootObject=None, rootLabel=None,
                 rootIsNamespace=True, intro='', locals=None, 
                 InterpClass=None, *args, **kwds):
        """Create a PyCrust Crust instance."""
        wx.wxSplitterWindow.__init__(self, parent, id, pos, size, style, name)
        self.shell = Shell(parent=self, introText=intro, 
                           locals=locals, InterpClass=InterpClass, 
                           *args, **kwds)
        if rootObject is None:
            rootObject = self.shell.interp.locals
        self.notebook = wx.wxNotebook(parent=self, id=-1)
        self.shell.interp.locals['notebook'] = self.notebook
        self.filling = Filling(parent=self.notebook, 
                               rootObject=rootObject, 
                               rootLabel=rootLabel, 
                               rootIsNamespace=rootIsNamespace)
        # Add 'filling' to the interpreter's locals.
        self.shell.interp.locals['filling'] = self.filling
        self.notebook.AddPage(page=self.filling, text='Namespace', select=True)
        self.calltip = Calltip(parent=self.notebook)
        self.notebook.AddPage(page=self.calltip, text='Calltip')
        self.sessionlisting = SessionListing(parent=self.notebook)
        self.notebook.AddPage(page=self.sessionlisting, text='Session')
        self.dispatcherlisting = DispatcherListing(parent=self.notebook)
        self.notebook.AddPage(page=self.dispatcherlisting, text='Dispatcher')
        from wxd import wx_
        self.wxdocs = Filling(parent=self.notebook, 
                              rootObject=wx_,
                              rootLabel='wx', 
                              rootIsNamespace=False,
                              static=True)
        self.notebook.AddPage(page=self.wxdocs, text='wxPython Docs')
        from wxd import stc_
        self.stcdocs = Filling(parent=self.notebook, 
                               rootObject=stc_.StyledTextCtrl,
                               rootLabel='StyledTextCtrl', 
                               rootIsNamespace=False,
                               static=True)
        self.notebook.AddPage(page=self.stcdocs, text='StyledTextCtrl Docs')
        self.SplitHorizontally(self.shell, self.notebook, 300)
        self.SetMinimumPaneSize(1)


class Calltip(wx.wxTextCtrl):
    """Text control containing the most recent shell calltip."""

    def __init__(self, parent=None, id=-1):
        import dispatcher
        style = wx.wxTE_MULTILINE | wx.wxTE_READONLY | wx.wxTE_RICH2
        wx.wxTextCtrl.__init__(self, parent=parent, id=id, style=style)
        self.SetBackgroundColour(wx.wxColour(255, 255, 232))
        dispatcher.connect(receiver=self.display, signal='Shell.calltip')

    def display(self, calltip):
        """Receiver for Shell.calltip signal."""
        self.SetValue(calltip)


class SessionListing(wx.wxTextCtrl):
    """Text control containing all commands for session."""

    def __init__(self, parent=None, id=-1):
        import dispatcher
        style = wx.wxTE_MULTILINE | wx.wxTE_READONLY | \
                wx.wxTE_RICH2 | wx.wxTE_DONTWRAP
        wx.wxTextCtrl.__init__(self, parent=parent, id=id, style=style)
        dispatcher.connect(receiver=self.push, signal='Interpreter.push')

    def push(self, command, more):
        """Receiver for Interpreter.push signal."""
        if command and not more:
            self.SetInsertionPointEnd()
            start, end = self.GetSelection()
            if start != end:
                self.SetSelection(0, 0)
            self.AppendText(command + '\n')


class DispatcherListing(wx.wxTextCtrl):
    """Text control containing all dispatches for session."""

    def __init__(self, parent=None, id=-1):
        import dispatcher
        style = wx.wxTE_MULTILINE | wx.wxTE_READONLY | \
                wx.wxTE_RICH2 | wx.wxTE_DONTWRAP
        wx.wxTextCtrl.__init__(self, parent=parent, id=id, style=style)
        dispatcher.connect(receiver=self.spy)

    def spy(self, signal, sender):
        """Receiver for Any signal from Any sender."""
        text = '%r from %s' % (signal, sender)
        self.SetInsertionPointEnd()
        start, end = self.GetSelection()
        if start != end:
            self.SetSelection(0, 0)
        self.AppendText(text + '\n')


class CrustFrame(wx.wxFrame, ShellMenu):
    """Frame containing all the PyCrust components."""

    name = 'PyCrust Frame'
    revision = __revision__

    def __init__(self, parent=None, id=-1, title='PyCrust', 
                 pos=wx.wxDefaultPosition, size=wx.wxDefaultSize, 
                 style=wx.wxDEFAULT_FRAME_STYLE, 
                 rootObject=None, rootLabel=None, rootIsNamespace=True, 
                 locals=None, InterpClass=None, *args, **kwds):
        """Create a PyCrust CrustFrame instance."""
        wx.wxFrame.__init__(self, parent, id, title, pos, size, style)
        intro = 'PyCrust %s - The Flakiest Python Shell' % VERSION
        intro += '\nSponsored by Orbtech - '
        intro += 'Your source for Python programming expertise.'
        self.CreateStatusBar()
        self.SetStatusText(intro.replace('\n', ', '))
        import images
        self.SetIcon(images.getPyCrustIcon())
        self.crust = Crust(parent=self, intro=intro, 
                           rootObject=rootObject, 
                           rootLabel=rootLabel, 
                           rootIsNamespace=rootIsNamespace, 
                           locals=locals, 
                           InterpClass=InterpClass, *args, **kwds)
        # Override the filling so that status messages go to the status bar.
        self.crust.filling.tree.setStatusText = self.SetStatusText
        # Override the shell so that status messages go to the status bar.
        self.crust.shell.setStatusText = self.SetStatusText
        # Fix a problem with the sash shrinking to nothing.
        self.crust.filling.SetSashPosition(200)
        self.shell = self.crust.shell
        self.createMenus()
        wx.EVT_CLOSE(self, self.OnCloseWindow)
        # Set focus to the shell editor.
        self.crust.shell.SetFocus()

    def OnCloseWindow(self, event):
        self.crust.shell.destroy()
        self.Destroy()


